# -*- coding: latin-1 -*-


import sys
sys.path.insert(0, '..')

from pynotebook.nbview import TextModel, WXTextView, Cell
from pynotebook.textformat import fromtext
import wx



examples = r"""[In 0]:
import matplotlib
matplotlib.use('Agg')
from pylab import *

if matplotlib.__version__ >= '1.4':
    matplotlib.style.use('ggplot')


[In 0]:
fig = plt.figure(facecolor='white')
r = np.arange(0, 3.0, 0.01)
theta = 2 * np.pi * r

ax = plt.subplot(111, polar=True)
ax.plot(theta, r, color='r', linewidth=3)
ax.set_rmax(2.0)
ax.grid(True)

ax.set_title("A line plot on a polar axis", va='bottom')
output(fig)

[In 0]:
fig # ... and again, this time without "output"!
[In 0]:
from numpy import cos
alpha = 0.7
phi_ext = 2 * pi * 0.5

def flux_qubit_potential(phi_m, phi_p):
    return 2 + alpha - 2 * cos(phi_p)*cos(phi_m) - alpha * cos(phi_ext - 2*phi_p)

phi_m = linspace(0, 2*pi, 100)
phi_p = linspace(0, 2*pi, 100)
X,Y = meshgrid(phi_p, phi_m)
Z = flux_qubit_potential(X, Y).T

fig, ax = plt.subplots()
fig.set_facecolor('white')

p = ax.pcolor(X/(2*pi), Y/(2*pi), Z, cmap=cm.RdBu, vmin=abs(Z).min(), vmax=abs(Z).max())
cb = fig.colorbar(p, ax=ax)
output(fig)
[In 0]:
fig, ax = plt.subplots(facecolor='white')
cnt = contour(Z, cmap=cm.RdBu, vmin=abs(Z).min(), vmax=abs(Z).max(), extent=[0, 1, 0, 1])
output(fig)
"""


def demo_00():
    app = wx.App(redirect=False)
    model = fromtext(examples, Cell=Cell)

    frame = wx.Frame(None)
    win = wx.Panel(frame, -1)
    view = WXTextView(win, -1, style=wx.SUNKEN_BORDER)
    view.model = model
    box = wx.BoxSizer(wx.VERTICAL)
    box.Add(view, 1, wx.ALL|wx.GROW, 1)
    win.SetSizer(box)
    win.SetAutoLayout(True)

    frame.Show()

    if 0:
        from pynotebook.wxtextview import testing
        testing.pyshell(locals())
    app.MainLoop()


if __name__ == '__main__':
    demo_00()



# -*- coding: latin-1 -*-

# Am 07.2.16 von wxtextview/demo/notebook.py übertragen 

from .textmodel import create_style
from .textmodel.treebase import insert
from .textmodel.texeltree import Characters, grouped, \
    defaultstyle, NULL_TEXEL
from .textmodel.textmodel import TextModel, dump_range
from .wxtextview.boxes import Box, VGroup, VBox, Row, Rect, check_box, NewlineBox, \
    TextBox, extend_range_seperated, replace_boxes, get_text
from .wxtextview.simplelayout import create_paragraphs, Paragraph
from .wxtextview.wxdevice import WxDevice
from .wxtextview.testdevice import TESTDEVICE
from .wxtextview.wxtextview import WXTextView as _WXTextView
from .wxtextview.simplelayout import Builder as _Builder

from .nbtexels import Cell, find_cell, mk_textmodel, NotFound
from .clients import ClientPool
from .pyclient import PythonClient
from .nbstream import Stream
import string
import wx



defaultstyle['temp'] = False 

promptstyle = create_style(
    textcolor = 'blue',
    weight = 'bold'
    )

sepwidth = 20000 # a number which is just larger than the textwidth

def is_temp(model, i):
    return model.get_style(i)['temp']



class ParagraphStack(VGroup):
    def create_group(self, l):
        return VGroup(l, device=self.device)



class CellBox(Box):
    def __init__(self, inbox, outbox, number=0, device=None):
        # NOTE: Inbox and outbox should be PargraphStacks
        self.number = number
        if device is not None:
            self.device=device
        self.input = inbox
        self.output = outbox
        self.layout()

    def from_childs(self, childs):
        box = self.__class__(childs[0], childs[1], self.number, self.device)
        return [box]

    def __len__(self):
        return self.length

    def create_group(self, l):
        return VGroup(l, device=self.device)

    def iter_boxes(self, i, x, y):
        input = self.input
        output = self.output
        height = self.height
        j1 = i+1
        j2 = j1+len(input)
        yield j1, j2, x+80, y, input
        j1 = j2
        j2 = j1+len(output)
        y += input.height+input.depth
        yield j1, j2, x+80, y, output

    def layout(self):
        # compute w and h
        dh = h = w = 0
        for j1, j2, x, y, child in self.iter_boxes(0, 0, 0):
            w = max(x+child.width, w)
            h = max(y+child.height, h)
            dh = max(y+child.height+child.depth, dh)
        self.width = w
        self.height = h
        self.depth = dh-h
        self.length = j2

    def __repr__(self):
        return self.__class__.__name__+'(%s, %s)' % \
            (repr(self.input),
             repr(self.output))

    def get_index(self, x, y):
        if y<3:
            return 0
        elif y>=self.height-2:
            return len(self)
        return Box.get_index(self, x, y)

    def extend_range(self, i1, i2):
        for i in (0, len(self.input), len(self)-1):
            if i1<= i<i2:
                #print "empty in i1..i2", i, (i1, i2)
                #self.dump_boxes(0, 0, 0)
                return min(i1, 0), max(i2, len(self))
        return extend_range_seperated(self, i1, i2)

    def draw(self, x, y, dc, styler):
        a, b = list(self.iter_boxes(0, x, y))
        styler.set_style(promptstyle)
        n = self.number or ''
        dc.DrawText("In[%s]:" % n, x, a[3])
        dc.DrawText("Out[%s]:" % n, x, b[3])
        Box.draw(self, x, y, dc, styler)

    def draw_selection(self, i1, i2, x, y, dc):
        if i1<=0 and i2>=self.length:
            self.device.invert_rect(x, y, self.width, self.height, dc)
        else:
            Box.draw_selection(self, i1, i2, x, y, dc)

    def responding_child(self, i, x0, y0):
        # Index position n+1 usually is managed by a child object. We
        # want the next object to be responsible, so we have to change
        # the return behaviour.
        if i == len(self):
            return None, i, x0, y0 # None => kein Kind kümert sich darum
        return Box.responding_child(self, i, x0, y0)

    def get_cursorrect(self, i, x0, y0, style):
        child, j, x1, y1 = self.responding_child(i, x0, y0)
        if child is not None:
            return child.get_cursorrect(i-j, x1, y1, style)
        return self.get_rect(i, x0, y0)

    def get_rect(self, i, x0, y0):
        child, j, x1, y1 = self.responding_child(i, x0, y0)
        if child is not None:
            return child.get_rect(i-j, x1, y1)
        if i == 0:
            return Rect(x0, y0, sepwidth, y0+2)
        assert i == len(self)
        h = self.height
        return Rect(x0, y0+h, sepwidth, y0+h+2)



def get_update_range(box, i1, i2):
    # Extend i1, i2 to the index range to be updated. E.g. a change in
    # an input field will lead to an update of the whole input
    # field. It is assumed, that extend range has been called before.

    if i2<=0 or i1>=len(box):
        return i1, i2
    if isinstance(box, CellBox):
        (j1, j2, inbox), (k1, k2, outbox) = box.iter_childs()
        if k1<=i1<=i2<=k1:    
            return k1, k2
        if j1<=i1<=i2<=j1:
            return j1, j2
        return min(i1, 0), max(i2, len(box))
    for j1, j2, child in box.iter_childs():
        if i1 < j2 and j1 < i2: # overlap
            k1, k2 = get_update_range(child, i1-j1, i2-j1)
            i1 = min(i1, k1+j1)
            i2 = max(i2, k2+j1)
    return i1, i2
            


class BitmapBox(Box):
    def __init__(self, bitmap, device=None):
        if device is not None:
            self.device = device
        self.bitmap = bitmap
        self.width, self.height = bitmap.Size
        self.depth = 0

    def __len__(self):
        return 1

    def iter_boxes(self, i, x, y):
        if 0: yield 0,0,0

    def draw(self, x, y, dc, styler):
        dc.DrawBitmap(self.bitmap, x, y, useMask=False)

    def get_index(self, x, y):
        if x>self.width/2.0:
            return 1
        return 0

    def draw_selection(self, i1, i2, x, y, dc):
        self.device.invert_rect(x, y, self.width, self.height, dc)



class Builder(_Builder):
    _has_temp = False # indicates whether the change we are updating
                      # to was caused by print_temp

    def __init__(self, model, clients=None, device=TESTDEVICE, maxw=0):
        if clients is None:
            clients = ClientPool()
        self._clients = clients
        _Builder.__init__(self, model, device, maxw)

    def create_paragraphs(self, texel, i1, i2, add_newline=False):
        boxes = self.create_boxes(texel, i1, i2)
        if add_newline:
            boxes = boxes+(self.NewlineBox(device=self.device),)
        if self._maxw:
            maxw = max(100, self._maxw-80)
        else:
            maxw = 0
        l = create_paragraphs(
            boxes, maxw = maxw,
            Paragraph = self.Paragraph,
            device = self.device)
        return l

    def create_parstack(self, texel, add_newline=False):
        l = self.create_paragraphs(texel, 0, len(texel), 
                                   add_newline=add_newline)
        return ParagraphStack(l, device=self.device)

    def rebuild(self):
        model = self.model
        boxes = self.create_all(model.texel)
        self._layout = VGroup(boxes, device=self.device)

    def rebuild_part(self, i1, i2, n):
        # $n$ is the size change. Positive $n$ means inserting, negativ
        # means removal. The new size is i1..i2+n.
        layout = self._layout
        model = self.model
        texel = model.texel

        # Did we insert temp text? If yes, temp should be excluded
        # from fontification.

        if n>0: 
            self._has_temp = is_temp(self.model, i1)
            self._temp_range = i1, i1+n
        else:
            self._has_temp = False

        #print "rebuild_part: i1, i2=", i1, i2, "n=", n
        #dump_range(texel, i1, i2)

        j1, j2 = layout.extend_range(i1, i2)
        k1, k2 = get_update_range(layout, j1, j2)
        #print "k1, k2=", k1, k2

        stuff = self.create_boxes(texel, k1, k2+n)
        l = replace_boxes(layout, k1, k2, stuff)
        self._layout = self.grouped(l)
        return self._layout
        
    ### Handlers
    def Cell_handler(self, texel, i1, i2):
        assert i2<=len(texel)
        #print "Cell handler: (i1, i2)=", (i1, i2)
        #dump_range(texel, 0, len(texel))
        
        (j1, j2, inp), (k1, k2, outp) = texel.iter_childs()
        if i1 < j2 and j1 < i2: 
            client = self._clients.get_matching(texel)
            inputfield = texel.input
            if self._has_temp:
                t1, t2 = self._temp_range
                i0, tmp = find_cell(self.model.texel, t1)
                assert tmp is texel
                i0 += 1 
                model = mk_textmodel(inputfield)
                old = model.remove(t1-i0, t2-i0)
                tmp = mk_textmodel(client.colorize(model.texel))
                model.insert(t1-i0, old)
                colorized = model.texel
            else:
                colorized = client.colorize(inputfield)
            inbox = self.create_parstack(colorized, add_newline=True)
        if i1 < k2 and k1 < i2: 
            outbox = self.create_parstack(texel.output, add_newline=True)

        if j1<=i1<=i2<=k1:
            assert j1 == i1 and i2 == j2+1
            return [inbox]

        if k1<=i1<=i2<=k2+1:
            assert k1 == i1 and i2 == k2+1
            return [outbox]

        assert i1 == 0 and i2 == k2+1
        cell = CellBox(inbox, outbox, number=texel.number,
                       device=self.device)
        return [cell]

    def Plot_handler(self, texel, i1, i2):
        return [PlotBox(device=self.device)]

    def BitmapRGB_handler(self, texel, i1, i2):
        w, h = texel.size
        bitmap = wx.BitmapFromBuffer(w, h, texel.data)
        return [BitmapBox(bitmap, device=self.device)]

    def BitmapRGBA_handler(self, texel, i1, i2):
        w, h = texel.size
        im = wx.ImageFromData(w, h, texel.data)
        im.SetAlphaBuffer(texel.alpha)
        bitmap = wx.BitmapFromImage(im)
        return [BitmapBox(bitmap, device=self.device)]

    ### Signals
    def properties_changed(self, i1, i2):
        return self.rebuild_part(i1, i2, 0)

    def inserted(self, i, n):
        return self.rebuild_part(i, i, n)

    def removed(self, i, n):
        return self.rebuild_part(i, i+n, -n)



def common(s1, s2):
    # Helper: find the common part of two strings. Needed by
    # completer.
    r = ''
    i = 0
    try:
        while s1[i] == s2[i]:
            i += 1
    except IndexError:
        pass
    return s1[:i]



class WXTextView(_WXTextView):
    temp_range = (0, 0)
    Cell = Cell
    def __init__(self, parent, id=-1,
                 pos=wx.DefaultPosition, size=wx.DefaultSize, style=0):
        self.init_clients()
        _WXTextView.__init__(self, parent, id=id, pos=pos, size=size,
                             style=style)
        self.actions[(wx.WXK_TAB, False, False)] = 'complete'
        self.actions[(wx.WXK_RETURN, False, False)] = 'insert_newline_indented'

    def init_clients(self):
        self._clients = ClientPool()        
        self._clients.register(PythonClient())        

    _resize_pending = False
    _new_size = None
    def on_size(self, event):
        # Note that resize involves computing all line breaks and is
        # therefore a very costly operation. We therefore try to avoid
        # unnecessary resize events.
        self._new_size = event.Size
        if self._resize_pending:
            return
        self._resize_pending = True
        wx.CallAfter(self._adjust_size)
        
    def _adjust_size(self):
        self._resize_pending = False
        maxw = self._new_size[0]
        if maxw == self._maxw:
            return
        self.set_maxw(maxw)
        self.keep_cursor_on_screen()

    def create_builder(self):
        return Builder(
            self.model,
            clients=self._clients,
            device=WxDevice(),
            maxw=self._maxw)

    def print_temp(self, text):
        new = TextModel(text)
        new.set_properties(0, len(new), textcolor='blue', temp=True)
        i = self.index
        self.model.insert(i, new)
        j1, j2 = self.temp_range
        if j1 == j2:
            self.temp_range = i, i+len(new)
        else:
            self.temp_range = j1, i+len(new)

    def clear_temp(self):
        j1, j2 = self.temp_range
        if j1 != j2:
            self.model.remove(j1, j2)
            self.temp_range = j1, j1

    def has_temp(self):
        j1, j2 = self.temp_range
        return not j1 == j2

    def get_word(self, j):
        model = self.model
        row, col = model.index2position(j)
        text = model.get_text(model.linestart(row), j)
        i = len(text)-1
        while i>=0 and text[i] in string.letters+string.digits+"_.":
            i -= 1
        if j == i:
            return ''
        return text[i+1:]

    def handle_action(self, action, shift=False):
        if action != 'complete':
            self.clear_temp()
            return _WXTextView.handle_action(self, action, shift)
        try:
            i0, cell = self.find_cell()
        except NotFound:
            return
        index = self.index
        if index <= i0 or index >= i0+len(cell):
            return
        if self.has_temp():
            self.clear_temp()
            maxoptions = 2000
        else:
            maxoptions = 200
        word = self.get_word(index)
        client = self._clients.get_matching(cell)
        options = client.complete(word, maxoptions)
        if not options:
            self.print_temp( "\n[No completion]\n")
        else:
            completion = reduce(common, options)[len(word):]
            if completion and len(options) != maxoptions:
                self.model.insert_text(index, completion)
                info = self._remove, index, index+len(completion)
                self.add_undo(info)
                index += len(completion)
            else:
                options = list(sorted(options))
                s = ', '.join(options)
                s = s.replace('(', '') # I don't like the bracket
                if len(options) == maxoptions:
                    s += ' ... '
                self.print_temp('\n'+s+'\n')
        self.index = index

    def on_char(self, event):
        keycode = event.GetKeyCode()
        ctrl = event.ControlDown()
        shift = event.ShiftDown()
        if keycode == wx.WXK_RETURN and shift and not ctrl:
            try:
                self.execute()
            except NotFound:
                pass
        else:
            _WXTextView.on_char(self, event)

    def find_cell(self):
        return find_cell(self.model.texel, self.index)

    def execute(self):
        i0, cell = self.find_cell()
        n = len(cell)
        client = self._clients.get_matching(cell)
        stream = Stream()
        client.execute(cell.input, stream.output)
        new = self.Cell(cell.input, stream.model.texel, client.counter)

        assert i0>=0
        assert i0+n<=len(self.model)
        infos = []
        infos.append(self._remove(i0, i0+n))
        self.model.insert(i0, mk_textmodel(new))
        infos.append((self._remove, i0, i0+len(new)))
        self.add_undo(infos) 
        self.adjust_viewport()

    def insert(self, i, textmodel):
        needscell = True
        try:
            i0, cell = self.find_cell()
            if not (i == i0 or i == i0+len(cell)):
                needscell = False
        except NotFound:
            pass
        try:
            find_cell(textmodel.texel, 0)
            hascell = True
        except NotFound:
            hascell = False
        if needscell and not hascell:
            cell = self.Cell(NULL_TEXEL, NULL_TEXEL)
            self.model.insert(i, mk_textmodel(cell))
            info = self._remove, i, i+len(cell)
            self.add_undo(info)
            i = i+1
        _WXTextView.insert(self, i, textmodel)



def init_testing(redirect=True):
    app = wx.App(redirect=redirect)
    model = TextModel('')

    frame = wx.Frame(None)
    win = wx.Panel(frame, -1)
    view = WXTextView(win, -1, style=wx.SUNKEN_BORDER)
    view.model = model
    box = wx.BoxSizer(wx.VERTICAL)
    box.Add(view, 1, wx.ALL|wx.GROW, 1)
    win.SetSizer(box)
    win.SetAutoLayout(True)

    
    frame.Show()
    return locals()

def test_00():
    "cell"
    ns = init_testing(False)
    cell = Cell(Characters(u'1234567890'), Characters(u'abcdefghij'))
    assert len(cell.input) == 10
    assert len(cell.output) == 10
    assert len(cell) == 23

    texel = grouped(insert(cell, 1, [Characters(u'x')]))
    assert texel.get_text()[1:2] == u'x'


def test_02():
    "execute"
    return # XXX skip this for now
    ns = init_testing(False)
    cell = Cell(Characters(u'1+2'), Characters(u''))
    cell = cell.execute()
    assert cell.output.get_text() == '3'

    cell = Cell(Characters(u'for a in range(2):\n    print a'),
                Characters(u''))
    cell = cell.execute()
    assert cell.output.get_text() == u'0\n1\n'

    cell = Cell(Characters(u'asdsad'), Characters(u''))
    cell = cell.execute()
    #print repr(cell.output.get_text())
    assert cell.output.get_text() == u'  File "In[3]", line 1, ' \
        'in <module>\nNameError: name \'asdsad\' is not defined\n'

def test_03():
    "find_cell"
    tmp1 = TextModel(u'for a in range(3):\n    print a')
    tmp2 = TextModel(u'for a in range(10):\n    print a')
    cell1 = Cell(tmp1.texel, Characters(u''))
    cell2 = Cell(tmp2.texel, Characters(u''))

    model = TextModel('')
    model.insert(len(model), mk_textmodel(cell1))
    model.insert(len(model), mk_textmodel(cell2))

    assert find_cell(model.texel, 1) == (0, cell1)
    assert find_cell(model.texel, len(cell1)-1) == (0, cell1)

    assert find_cell(model.texel, len(cell1)) == (len(cell1), cell2)
    assert find_cell(model.texel, len(cell1)+5) == (len(cell1), cell2)


def test_04():
    "copy cells"
    model = TextModel('')
    tmp = TextModel(u'for a in range(5):\n    print a')
    cell = Cell(tmp.texel, Characters(u''))
    model.insert(len(model), mk_textmodel(cell))
    tmp = model.copy(0, len(model))
    model.insert(0, tmp)

def test_05():
    "CellBox"
    empty = VGroup([])
    cell1 = CellBox(empty, empty)
    check_box(cell1.input)
    check_box(cell1)
    #cell1.dump_boxes(0, 0, 0)

    cell2 = CellBox(empty, empty)
    check_box(cell2)
    stack = VGroup([cell1, cell2])
    check_box(stack)

    stack = VGroup([])
    check_box(stack)

    t1 = TextBox("0123456789")
    t2 = TextBox("abcdefghij")
    t3 = TextBox("xyz")
    NL = NewlineBox()
    p1 = Paragraph([Row([t1, NL])])
    cell1 = CellBox(
        VGroup([Paragraph([Row([t1, NL])]), Paragraph([Row([t2, NL])]),]), 
        Paragraph([Row([t3, NL])]))

    (j1, j2, inp), (k1, k2, outp) = cell1.iter_childs()
    assert (j1, j2) == (1, 23) # input box

    cell2 = CellBox(Row([t3, NL]), empty)

    g = VGroup([cell1, cell2])


def test_06():
    "output"
    buf = TextBuffer()
    inter = SimpleInterpreter()
    inter.execute("output(1)", buf.output)
    assert not buf.stderr
    inter.execute("output(1.1)", buf.output)
    assert not buf.stderr
    inter.execute(u"output('ö')", buf.output)
    assert not buf.stderr

    code = '''import matplotlib.pyplot as plt
plt.plot([1,2,3,4])
plt.ylabel('some numbers')
figure = plt.figure()
output(figure)
'''
    buf = TextBuffer()
    inter.execute(code, buf.output)
    figure = inter.namespace['figure']

    assert has_classname(figure, 'matplotlib.figure.Figure')
    assert buf.stdout == 'Graphics ---'
    assert not buf.stderr


def test_10():
    "Factory"
    ns = init_testing(False)
    cell = Cell(Characters(u'a'), Characters(u'b'))
    factory = Builder(TextModel(''))
    factory._clients.register(PythonClient())
    boxes = factory.create_all(cell)
    assert len(boxes) == 1
    cellbox = boxes[0]
    assert len(cellbox) == 5
    assert len(cell) == 5

    check_box(cellbox)
    check_box(cellbox.input)
    check_box(cellbox.output)
    return ns


def test_11():
    ns = init_testing(redirect=True)
    model = ns['model']
    model.remove(0, len(model))
    tmp = TextModel(u'for a in range(16):\n    print a')
    cell = Cell(tmp.texel, Characters(u''))
    model.insert(len(model), mk_textmodel(cell))

    assert model.index2position(0) == (0, 0)
    assert model.index2position(1) == (1, 0)
    cell = model.texel

    assert find_cell(model.texel, 1) == (0, cell)

    view = ns['view']
    view.index = 1
    #print model.texel
    view.execute()

    check_box(view.builder._layout, model.texel)

    #view.layout.dump_boxes(0, 0, 0)
    #assert inside_cell(view.layout, 68, 68)

    model.insert_text(68, u'x')
    assert model.get_text()[65:71] == '14\nx15'
    #view.layout.dump_boxes(0, 0, 0)
    
    model.remove(68, 69)
    #view.layout.dump_boxes(0, 0, 0)
    assert model.get_text()[65:70] == '14\n15'

    model.insert(0, mk_textmodel(cell))
    model.remove(0, len(cell))

    # insert new cell, insert input
    view.insert(0, TextModel("a=1"))

    # insert output
    view.insert(5, TextModel("x"))

    # remove the complete input-part
    model.remove(1, 4)

    # remove the complete output-part
    model.remove(2,3)
    
    return ns

def test_12():
    ns = init_testing(True)
    model = ns['model']
    model.remove(0, len(model))
    tmp = TextModel(u'for a in range(5):\n    print a')
    cell = Cell(tmp.texel, NULL_TEXEL)
    model.insert(len(model), mk_textmodel(cell))

    assert find_cell(model.texel, 1) == (0, cell)

    view = ns['view']
    view.index = 1
    return ns


def test_13():
    ns = init_testing(True)
    model = ns['model']
    model.remove(0, len(model))
    tmp = TextModel(u'''import matplotlib.pyplot as plt
figure = plt.figure(
    facecolor='white',
    figsize=(3, 2.5))
figure.set_frameon(False)
plot = figure.add_subplot ( 111 )
plt.plot([1,2,3,4])
plt.ylabel('some numbers')
output(figure)''')
    cell = Cell(tmp.texel, NULL_TEXEL)
    model.insert(len(model), mk_textmodel(cell))

    assert find_cell(model.texel, 1) == (0, cell)

    view = ns['view']
    view.index = 1
    view.execute()
    return ns

def test_14():
    "cell cursor"
    ns = init_testing(False)
    model = ns['model']
    model.remove(0, len(model))
    tmp = TextModel(u'for a in range(5):\n    print a')
    cell = Cell(tmp.texel, NULL_TEXEL)
    model.insert(len(model), mk_textmodel(cell))

    assert find_cell(model.texel, 1) == (0, cell)

    view = ns['view']
    view.index = 1
    layout = view.builder.get_layout()
    r1 = layout.get_rect(0, 0, 0)
    assert r1.x2-r1.x1 == sepwidth

    r2 = layout.get_cursorrect(0, 0, 0, defaultstyle)
    r3 = layout.get_cursorrect(len(model), 0, 0, defaultstyle)
    assert r3.x2-r3.x1 == sepwidth
    assert r3.y1 > r2.y1

def test_15():
    "get_word, print_temp, remove_temp"
    ns = init_testing(False)
    model = ns['model']
    tmp = TextModel(u'for a in range(5):\n    print a')
    cell = Cell(tmp.texel, NULL_TEXEL)
    model.insert(len(model), mk_textmodel(cell))
    view = ns['view']
    assert view.get_word(12) == 'ra'

    text = model.get_text()
    view.index = 12
    view.print_temp('\n[raise range raw_input]\n')
    #print model.get_text()

    view.clear_temp()
    assert model.get_text() == text


    
def demo_00():
    from .wxtextview import testing
    ns = test_11()
    testing.pyshell(ns)
    ns['app'].MainLoop()

def demo_01():
    from .wxtextview import testing
    ns = test_13()
    testing.pyshell(ns)
    ns['app'].MainLoop()





# -*- coding: latin-1 -*-


from ..textmodel.viewbase import ViewBase, overridable_property
from ..textmodel.modelbase import Model
from ..textmodel.textmodel import dump_range
from ..textmodel import TextModel


debug = 0
#debug = 1

def undo(info):
    if callable(info[0]):
        func = info[0]
        args = info[1:]    
        redo = apply(func, args)
    else:
        redo = []
        for child in reversed(info):
            redo.append(undo(child))
    return redo


class TextView(ViewBase, Model):
    index = overridable_property('index')
    _index = 0
    selection = overridable_property('selection') # NOTE: i2 can also
                                                  # be smaller than
                                                  # i1!
    _selection = None
    maxw = overridable_property('maxw')
    _maxw = 0
    _scrollrate = 10, 10
    _TextModel = TextModel
    def __init__(self):
        ViewBase.__init__(self)
        self._undoinfo = []
        self._redoinfo = []
        self.set_model(self._TextModel(''))
        assert self.layout is not None

    def create_builder(self):
        pass

    def set_model(self, model):
        ViewBase.set_model(self, model)
        self.builder = self.create_builder()
        self.rebuild()

    def rebuild(self):
        self.builder.rebuild()
        self.layout = self.builder.get_layout()
        assert self.layout is not None

    def join_undo(self, info2, info1):
        # we are joining similar undo entries
        if 0:
            print "join"
            print "info1=", info1
            print "info2=", info2
        if info1[0] == info2[0]:
            if info1[0] == self._remove:
                i1, i2 = info1[1:]
                j1, j2 = info2[1:]
                if i2 == j1 and j2-i1<10:
                    return [(self._remove, i1, j2)]
            elif info1[0] == self._undo_remove:
                s, i1, i2 = info1[1:]
                t, j1, j2 = info2[1:]
                #print "i1, i2:", i1, i2
                #print "j1, j2:", j1, j2
                if j2 == i1 and i2-j1<10:
                    t.insert(len(t), s)
                    return [(self._undo_remove, t, j1, i2)]
                if i1 == j1 and i2-j1<10:
                    t.insert(0, s)
                    #print "remove:", repr(t.get_text()), i1, i2+j2-j1
                    return [(self._undo_remove, t, i1, i2+j2-j1)]
                    
        return [info2, info1]

    def undo(self):
        if len(self._undoinfo) > 0:
            self.add_redo(undo(self._undoinfo[0]))
            del self._undoinfo[0]
                
    def add_undo(self, info, clear_redo = 1):
        if info is not None:
            if len(self._undoinfo):
                joined = self.join_undo(info, self._undoinfo[0])
                self._undoinfo = joined + self._undoinfo[1:]
            else:
                self._undoinfo.insert(0, info)
            if clear_redo:
                self._redoinfo = []
            self.notify_views('undo_changed')

    def redo(self):
        if len(self._redoinfo) > 0:
            self.add_undo(undo(self._redoinfo[0]), 0)
            del self._redoinfo[0]

    def add_redo(self, info):
        # Internal method: add a single redo info
        self._redoinfo.insert(0, info)
        self.notify_views('undo_changed')

    def insert(self, i, textmodel):
        self.model.insert(i, textmodel)
        self.index = i+len(textmodel)
        info = self._remove, i, i+len(textmodel)
        self.add_undo(info)

    def remove(self, i1, i2):
        info = self._remove(i1, i2)
        self.add_undo(info)
        self.index = i1

    def _remove(self, i1, i2):
        old = self.model.remove(i1, i2)
        self.index = i1
        return self._undo_remove, old, i1, i2

    def _undo_remove(self, old, i1, i2):
        self.model.insert(i1, old)
        self.index = i2
        return self._remove, i1, i2

    def set_maxw(self, maxw):
        if maxw == self._maxw:
            return
        self._maxw = maxw
        self.builder.set_maxw(maxw)
        self.layout = self.builder.get_layout()
        self.Refresh()
        self.notify_views('maxw_changed')

    def compute_index(self, x, y):
        if y >= self.layout.height:
            return len(self.model)-1
        if y < 0:
            return 0
        return self.layout.get_index(x, y)

    def handle_action(self, action, shift=False):
        model = self.model
        index = self.index
        layout = self.layout
        if index == len(model):
            if index == 0:
                style = {} 
            else:
                style = model.get_style(index-1)
        else:
            style = model.get_style(index)
        row, col = self.current_position()
        rect = layout.get_rect(index, 0, 0)
        x = rect.x1
        y = rect.y1

        def del_selection():
            if self.has_selection():
                s1, s2 = sorted(self.selection)
                s1, s2 = layout.extend_range(s1, s2)
                self.remove(s1, s2)

        if action == 'dump_info':
            s1, s2 = sorted(self.selection)
            s1, s2 = layout.extend_range(s1, s2)
            dump_range(model.texel, s1, s2)
            row, col = model.index2position(index)
            print "index=", index
            print "row=", row
            print "col=", col
            
        elif action == 'move_word_end':
            i = index
            n = len(model)
            try:
                while not model.get_text(i, i+1).isalnum():
                    i = i+1
                while model.get_text(i, i+1).isalnum():
                    i = i+1
            except IndexError:
                i = n
            self.set_index(i, shift)
        elif action == 'move_right':
            self.set_index(index+1, shift)
        elif action == 'move_word_begin':
            i = index
            try:
                while not model.get_text(i-1, i).isalnum():
                    i = i-1
                while model.get_text(i-1, i).isalnum():
                    i = i-1
            except IndexError:
                i = 0
            self.set_index(i, shift)
        elif action == 'move_left':
            self.set_index(index-1, shift)
        elif action == 'move_paragraph_end':
            i = row
            try:
                while model.linelength(i) == 1:
                    i += 1
                while model.linelength(i) > 1:
                    i += 1
                self.move_cursor_to(i, 0, shift)
            except IndexError:
                self.set_index(len(model), shift)                    
        elif action == 'move_down':
            self.move_cursor_to(row+1, col, shift)
        elif action == 'move_paragraph_begin':
            i = row-1
            while i >= 0 and model.linelength(i) == 1:
                i -= 1
            while i >= 0 and model.linelength(i) > 1:
                i -= 1
            self.move_cursor_to(i+1, 0, shift)
        elif action == 'move_up':
            self.move_cursor_to(row-1, col, shift)
        elif action == 'move_line_start':
            self.set_index(model.linestart(row), shift)
        elif action == 'move_line_end':
            self.set_index(model.linestart(row)+model.linelength(row)-1, shift)
        elif action == 'move_page_down':
            width, height = self.GetClientSize()
            i = self.compute_index(x, y+height)
            self.set_index(i, shift)            
        elif action == 'move_page_up':
            width, height = self.GetClientSize()
            i = self.compute_index(x, y-height)
            self.set_index(i, shift)
        elif action == 'move_document_start':
            self.set_index(0, shift)
        elif action == 'move_document_end':
            self.set_index(len(model), shift)
        elif action == 'select_all':
            self.selection = (0, len(model))
        elif action == 'insert_newline':
            self.insert(index, self._TextModel('\n', **style))
        elif action == 'insert_newline_indented':
            i = model.linestart(row)
            s = model.get_text(i, index)
            l = s[:len(s)-len(s.lstrip())]
            self.insert(index, self._TextModel('\n'+l, **style))
        elif action == 'backspace':
            del_selection()
            i = self.index
            if i>0:
                j1, j2 = layout.extend_range(i-1, i)
                self.remove(j1, j2)
        elif action == 'copy':
            self.copy()
        elif action == 'paste':
            self.paste()
        elif action == 'cut':
            self.cut()
        elif action == 'delete':
            if self.has_selection():
                del_selection()
            else:
                i = self.index
                if i < len(self.model):
                    j1, j2 = layout.extend_range(i, i+1)
                    self.remove(j1, j2)
        elif action == 'undo':
            self.undo()
        elif action == 'redo':
            self.redo()
        elif action == 'del_word_left':
            # find the beginning of the word
            i = index
            try:
                while not model.get_text(i-1, i).isalnum():
                    i = i-1
                while model.get_text(i-1, i).isalnum():
                    i = i-1
            except IndexError:
                i = 0
            self.remove(i, index)
        else:                  
            #print keycode
            assert len(action) == 1 # single character
            del_selection()
            index = self.index
                
            s = self._TextModel(action, **style)  
            n = len(model)
            assert len(s) == 1
            self.insert(index, s)
        self.Refresh()

    def copy(self):
        raise NotImplemented()

    def paste(self):
        raise NotImplemented()

    def cut(self):
        raise NotImplemented()
         
    def select_word(self, x, y):
        i = self.layout.get_index(x, y)
        if i is None:
            return
        model = self.model
        n = len(model)
        try:
            while not model.get_text(i-1, i).isalnum():
                i = i-1
            while model.get_text(i-1, i).isalnum():
                i = i-1
        except IndexError:
            i = 0
        i1 = i
        i = i1
        try:
            while not model.get_text(i, i+1).isalnum():
                i = i+1
            while model.get_text(i, i+1).isalnum():
                i = i+1
        except IndexError:
            i = n
        i2 = i
        self.index = i2
        self.selection = (i1, i2)

    def refresh(self):
        raise NotImplemented()

    def check(self):
        from ..textmodel.treebase import is_root_efficient
        assert is_root_efficient(self.layout)

    ### Signals issued by model
    def properties_changed(self, model, i1, i2):
        self.builder.properties_changed(i1, i2)
        self.layout = self.builder.get_layout()
        self.refresh()

    def inserted(self, model, i, n):
        self.builder.inserted(i, n)
        self.layout = self.builder.get_layout()
        if debug:
            self.check()
        if i>= self.index:
            self.index += n
        if self._selection is not None:
            s1, s2 = self.selection
            if i >= s1:
                s1 += n
            if i >= s2:
                s2 += n
            self.selection = s1, s2
        self.refresh()

    def removed(self, model, i, text):
        self.builder.removed(i, len(text))
        self.layout = self.builder.get_layout()
        n = len(text)
        i1 = i
        i2 = i+n
        m = len(model)
        index = self.index
        if index >= i2:
            self.index = index-n
        elif index > i1:
            self.index = i1
        if self._selection is not None:
            s1, s2 = self.selection
            if s1 >= i2:
                s1 -= n
            elif s1 > i1:
                s1 = i1
            if s2 >= i2:
                s2 -= n
            elif s2 > i1:
                s2 = i1
            self.selection = min(s1, m), min(s2, m)
        self.refresh()

    def keep_cursor_on_screen(self):
        pass
        
    ### Index
    def set_index(self, index, extend=False, update=True):
        if index < 0:
            index = 0
        elif index > len(self.model):
            index = len(self.model)
        if index != self._index:
            self._index = index
            if extend:
                self.extend_selection()
            elif update:
                self.start_selection()
            self.adjust_viewport()
            self.refresh()
            self.notify_views('index_changed')

    def get_index(self):
        return self._index

    def current_position(self):
        # Returns the cursorposition as tuple (row, col)
        model = self.model
        i = self.index
        if model is None or i == 0:
            return 0, 0
        return model.index2position(i)

    def move_cursor_to(self, row, col, extend=False, update=True):
        # Moves cursor to (row, col). If this position is non existent
        # the next possible value is selected.

        # extend: extend selection
        # update: update selection 

        model = self.model
        row = max(0, min(row, model.nlines()-1))
        col = max(0, min(col, model.linelength(row)-1))
        self.set_index(model.position2index(row, col), extend, update)

    def get_selection(self):
        return self._selection

    def set_selection(self, selection):
        old = self._selection
        if selection == old:
            return
        if old is not None:
            i1, i2 = old
        self._selection = selection
        self.Refresh()
        self.notify_views('selection_changed')

    def has_selection(self):
        selection = self.selection
        if selection is None:
            return False
        return selection[0] != selection[1]

    def get_selected(self):
        # Returns a list of selected regions. So far only a
        # continguous region can be selected. In the futur this can
        # change (for example for tables). Box.extend_selection should
        # be renamed in Box.get_selection then.
        selection = self.selection
        if selection is None:
            return []
        s1, s2 = sorted(self.selection)
        return [self.layout.extend_range(s1, s2)]

    def start_selection(self):
        index = self.index
        self.selection = index, index
        
    def extend_selection(self):
        # Moves the selection endoint to index
        selection = self.selection
        index = self.index
        if selection is None:
            self.selection = index, index
        else:
            self.selection = selection[0], index
        
        
    



# -*- coding: latin-1 -*-

import wx


def invert_rect_INV(self, x, y, w, h, dc):
    dc.SetLogicalFunction(wx.INVERT)
    dc.SetBrush(wx.BLACK_BRUSH)
    dc.DrawRectangle(x, y, w, h)


def invert_rect_BLIT(self, x, y, w, h, dc):
    dc.Blit(x, y, w, h, dc, x, y, wx.SRC_INVERT)


def measure_win(self, text, style):
    font = wx.Font(style['fontsize'], wx.MODERN, wx.NORMAL, wx.NORMAL,
                   style['underline'], style['facename'])
    dc = wx.MemoryDC()
    dc.SetFont(font)
    w, h = dc.GetTextExtent(text)
    return w, h


def measure_mac(self, text, style):
    font = wx.Font(style['fontsize'], wx.MODERN, wx.NORMAL, wx.NORMAL,
                   style['underline'], style['facename'])
    gc = wx.GraphicsContext_CreateMeasuringContext()
    gc.SetFont(font)
    w, h = gc.GetTextExtent(text)
    return w, h


def measure_gtk(self, text, style):
    # GC return wrong font metric values in gtk! We therefore use the DC.  
    text = text.replace('\n', ' ') # replace newlines to avoid double lines
    font = wx.Font(style['fontsize'], wx.MODERN, wx.NORMAL, wx.NORMAL,
                   style['underline'], style['facename'])
    dc = wx.MemoryDC()
    dc.SetFont(font)
    w, h = dc.GetTextExtent(text)
    return w, h


def measure_parts_win(self, text, style):
    font = wx.Font(style['fontsize'], wx.MODERN, wx.NORMAL, wx.NORMAL,
                   style['underline'], style['facename'])
    dc = wx.MemoryDC()
    dc.SetFont(font)
    return dc.GetPartialTextExtents(text)


def measure_parts_gtk(self, text, style):
    font = wx.Font(style['fontsize'], wx.MODERN, wx.NORMAL, wx.NORMAL,
                   style['underline'], style['facename'])
    dc = wx.MemoryDC()
    dc.SetFont(font)
    return dc.GetPartialTextExtents(text)


def measure_parts_mac(self, text, style):
    font = wx.Font(style['fontsize'], wx.MODERN, wx.NORMAL, wx.NORMAL,
                   style['underline'], style['facename'])
    gc = wx.GraphicsContext_CreateMeasuringContext()
    gc.SetFont(font)
    return gc.GetPartialTextExtents(text)


class WxDevice:
    def intersects(self, dc, rect):
        r = wx.Rect(rect.x1, rect.y1, rect.x2-rect.x1, rect.y2-rect.y1)
        return dc.ClippingRect.Intersects(r)

    def draw_text(self, text, x, y, dc):
        dc.DrawText(text, x, y)

    if "msw" in wx.version():
        invert_rect = invert_rect_BLIT
        measure = measure_win
        measure_parts = measure_parts_win
        buffering = True
    elif "gtk" in wx.version():
        invert_rect = invert_rect_BLIT
        measure = measure_gtk
        measure_parts = measure_parts_gtk
        buffering = True
    elif 'mac' in wx.version():
        invert_rect = invert_rect_INV
        measure = measure_mac
        measure_parts = measure_parts_mac
        buffering = False





# -*- coding: latin-1 -*-



from ..textmodel import listtools
from ..textmodel import treebase
from ..textmodel.textmodel import TextModel
from ..textmodel.texeltree import NewLine, Group, Characters, defaultstyle
from .testdevice import TESTDEVICE
from .boxes import TextBox, NewlineBox, TabulatorBox, EmptyTextBox, \
    EndBox, check_box, Box



class Factory:
    TextBox = TextBox
    NewlineBox = NewlineBox
    TabulatorBox = TabulatorBox
    EndBox = EndBox

    def __init__(self, device=TESTDEVICE):
        self.device = device

    def get_device(self):
        return self.device

    ### Factory methods
    def create_all(self, texel):
        # Convenience method
        return self.create_boxes(texel, 0, len(texel))

    def create_boxes(self, texel, i1, i2):
        assert i1>=0
        assert i2<=len(texel)
        assert i1<=i2
        if i1 == i2:
            return () # XXX Why is this needed?
        name = texel.__class__.__name__+'_handler'
        handler = getattr(self, name)
        #print "calling handler", name, i1, i2
        l = handler(texel, i1, i2)
        try:
            assert listtools.calc_length(l) == i2-i1
        except:
            print "handler=", handler
            raise
        return tuple(l)
        
    def Group_handler(self, texel, i1, i2):
        r = []
        for j1, j2, child in texel.iter_childs():
            if i1 < j2 and j1 < i2: # overlapp
                r.extend(self.create_boxes(
                    child, max(0, i1-j1), min(i2, j2)-j1))
        return r

    def Characters_handler(self, texel, i1, i2):
        return [self.TextBox(texel.text[i1:i2], texel.style, self.device)]

    def NewLine_handler(self, texel, i1, i2):
        if texel.is_endmark:
            return [self.EndBox(texel.style, self.device)]
        return [self.NewlineBox(texel.style, self.device)] # XXX: Hmmmm

    def Tabulator_handler(self, texel, i1, i2):
        return [self.TabulatorBox(texel.style, self.device)]





class BuilderBase:
    """The builder is responsible for creating and updating the layout. 

    The methods build, insert, remove and update return a tree of boxes (=
    "layout").

    The length of the box tree is always the length of the model +1,
    because we add a special box ("end mark").

    """

    _layout = None

    def rebuild(self):
        # sets self._layout
        pass

    def get_layout(self):
        assert self._layout is not None
        return self._layout

    ### Signal handlers
    def properties_changed(self, i1, i2):
        pass

    def inserted(self, i, n):
        pass

    def removed(self, i, n):
        pass



def test_01():
    factory = Factory()
    boxes = factory.create_all(TextModel("123").texel)
    assert listtools.calc_length(boxes) == 3
    boxes = factory.create_all(TextModel("123\n567").get_xtexel())
    assert listtools.calc_length(boxes) == 8
    paragraphs = create_paragraphs(boxes)
    assert len(paragraphs) == 2
    assert len(paragraphs[0]) == 4
    assert len(paragraphs[1]) == 4
    assert listtools.calc_length(paragraphs) == 8

def test_02():
    "ParagraphStack"
    factory = Factory()
    texel = TextModel("123\n567\n").texel
    boxes = factory.create_all(texel)
    assert listtools.calc_length(boxes) == 8
    paragraphs = create_paragraphs(boxes)
    stack = ParagraphStack(paragraphs)
    assert check_box(stack, texel)

    texel = TextModel("123\n\n5\n67\n").texel
    boxes = factory.create_all(texel)
    assert listtools.calc_length(boxes) == 10
    paragraphs = create_paragraphs(boxes)
    stack = ParagraphStack(paragraphs)
    assert check_box(stack, texel)

    texel = TextModel("123\n").texel
    boxes = factory.create_all(texel)
    paragraphs = create_paragraphs(boxes)
    stack = ParagraphStack(paragraphs)
    par = stack.childs[-1]
    assert len(par) == 4
    assert len(stack) == 4

    if 0:
        for p in paragraphs:
            p.dump_boxes(0, 0, 0)

    assert stack.get_info(3, 0, 0)[-2:] == (3, 0)

    texel = TextModel("").texel
    boxes = factory.create_all(texel)
    paragraphs = create_paragraphs(boxes)
    stack = ParagraphStack(paragraphs)
    

def test_03():
    "Factory"
    factory = Factory()
    texel = TextModel("123\n\n567890 2 4 6 8 0\n").texel
    boxes = factory.create_all(texel)
    assert str(boxes) == "(TB('123'), NL, NL, TB('567890 2 4 6 8 0'), NL)"
    paragraphs = create_paragraphs(boxes)
    stack = ParagraphStack(paragraphs)
    assert str(tuple(stack.childs)) == "(Paragraph[Row[TB('123'), NL]], " \
        "Paragraph[Row[NL]], Paragraph[Row[TB('567890 2 4 6 8 0'), NL]])"

    # line break
    paragraphs = create_paragraphs(boxes, 5) 
    # Depends on the line break algorithm. Here breaking after space
    # is assumed.
    assert repr(paragraphs) == "[Paragraph[Row[TB('123'), NL]]," \
        " Paragraph[Row[NL]], Paragraph[Row[TB('56789')], Row[TB('0 2 ')], " \
        "Row[TB('4 6 ')], Row[TB('8 0'), NL]]]"


    texel = TextModel("123\t\t567890 2 4 6 8 0\n").texel
    
    boxes = factory.create_all(texel)

    factory.create_all(texel)
    paragraphs = create_paragraphs(boxes)

def test_04():
    "insert/remove"
    factory = Factory()
    model = TextModel("123\n\n567890 2 4 6 8 0")
    builder = Builder(model, maxw=0)
    builder.rebuild()
    layout = builder.get_layout()
    assert repr(layout) == "ParagraphStack[Paragraph[Row[TB('123'), " \
        "NL]], Paragraph[Row[NL]], Paragraph[Row[TB('567890 2 4 6 8 0'), " \
        "ENDMARK]]]"
    assert len(layout) == len(model)+1
    assert layout.height == 3

    ins = TextModel("xyz\n")
    model.insert(2, ins)
    builder.inserted(2, len(ins))
    assert len(builder._layout) == len(model)+1
    assert repr(builder._layout) == "ParagraphStack[Paragraph[Row[TB('12xyz'),"\
        " NL]], Paragraph[Row[TB('3'), NL]], Paragraph[Row[NL]], "\
        "Paragraph[Row[TB('567890 2 4 6 8 0'), ENDMARK]]]"
    assert builder._layout.height == 4
    model.remove(2, 2+len(ins))
    builder.removed(2, len(ins))
    assert len(builder._layout) == len(model)+1
    assert repr(builder._layout) == "ParagraphStack[Paragraph[Row[TB('123'), " \
        "NL]], Paragraph[Row[NL]], Paragraph[Row[TB('567890 2 4 6 8 0'), " \
        "ENDMARK]]]"
    assert builder._layout.height == 3

    factory = Factory()
    model = TextModel("123")
    builder = Builder(model, maxw=0)
    layout = builder._layout

    ins = TextModel("xyz\n")    
    i = len(model)
    model.insert(i, ins)
    builder.inserted(i, len(ins))
    
    for c in "abc":
        ins = TextModel(c)    
        i = len(model)
        model.insert(i, ins)
        builder.inserted(i, len(ins))
    assert str(builder._layout) == \
        "ParagraphStack[Paragraph[Row[TB('123xyz'), NL]], " \
        "Paragraph[Row[TB('abc'), ENDMARK]]]"


def test_05():
    device = TESTDEVICE
    model = TextModel("123\n\n567890 2 4 6 8 0")
    builder = Builder(model, maxw=0)
    
    def check(box):
        if not box.device is device:
            print box
        assert box.device is device
        if hasattr(box, 'iter'):
            for j1, j2, x1, y1, child in box.riter(0, 0, 0):
                check(child)
    check(builder.get_layout())

def test_06():
    model = TextModel("123\n")
    builder = Builder(model, maxw=0)
    layout = builder._layout
    assert layout.get_info(4, 0, 0)
    assert str(layout.get_info(3, 0, 0)) == "(NL, 0, 3, 0)"
    assert layout.get_info(3, 0, 0)[-2:] == (3, 0)
    assert layout.get_info(4, 0, 0)[-2:] == (0, 1)

def test_07():
    # Problem: when clicking right beside a row the cursor is inserted
    # in a wrong position.
    model = TextModel("123\n567")
    builder = Builder(model, maxw=0)
    layout = builder._layout
    assert layout.get_index(100, 0.5) == 3

def test_08():
    "linewrap"
    model = TextModel("aa bb cc dd ee")
    builder = Builder(model, maxw=0)
    builder.get_layout().dump_boxes(0, 0, 0)

    model = TextModel("aa bb cc dd ee")
    builder.set_maxw(5)
    builder.get_layout().dump_boxes(0, 0, 0)
    
    layout = builder.get_layout()
    for j1, j2, x, y, child in layout.iter(0, 0, 0):
        print j1, j2, x, y, child



# -*- coding: latin-1 -*-

class TestDevice:
    # Device is a interface layer to capsule platform dependent
    # graphics methods.

    buffering = False
    def measure(self, text, style):
        return len(text), 1 # Dummy for testing

    def measure_parts(self, text, style):
        return tuple(range(1, len(text)+1)) # Dummy for testing

    def intersects(self, dc, rect):
        return True

    def invert_rect(self, x, y, w, h, dc):
        pass

    def draw_text(self, text, x, y, dc):
        pass


TESTDEVICE = TestDevice()



# -*- coding: latin-1 -*-


from . import treebase
from copy import copy as shallow_copy
from .treebase import remove, copy, simple_insert, insert, groups, \
    depth, get_rightmost, get_leftmost, exchange_rightmost, \
    remove_leftmost, ungroup, strip, heal, is_homogeneous, \
    is_root_efficient, join, homogenize, maxdepth
from .treebase import grouped as _grouped
from .listtools import listjoin, calc_length


debug = 0 # no checks
#debug = 1 # do checks

def hash_style(style):
    return tuple(sorted(style.items()))

style_pool = {}
defaultstyle = {}
def create_style(**kwds):
    global style_pool
    style = defaultstyle.copy()
    style.update(kwds)
    key = hash_style(style)
    try:
        return style_pool[key]
    except KeyError:
        style_pool[key] = style
        return style

defaultstyle = create_style(
    textcolor='black',
    bgcolor='white',
    fontsize=10
    )


def updated_style(style, properties):
    new = style.copy()
    new.update(properties)
    return create_style(**new)


def _style_add((n1, style1), (n2, style2)):
    # used as argument for listjoin: listjoin(a, b, _style_add)
    if style1 is style2:
        return [(n1+n2, style1)]
    return [(n1, style1), (n2, style2)]



class Texel:
    """Baseclass of all text elements.

    Texts consist of texels (TExt-ELements) similar as pictures
    consist of pixels (PICture-ELements).

    All texels need to be derived from Texel. In addition they need to
    be derived from treebase.Element.
    """
    weights = treebase.Element.weights+(0,) # new weight: number of new lines
    functions = treebase.Element.functions+(sum,)

    def get_style(self, i):
        """Returns the style at index $i$"""
        return {}

    def get_text(self):
        """Returns an unicode string with all textual content.

           post:
               len(self) == len(__return__)
        """
        return u' '

    def get_styles(self, i1, i2):
        """Returns all format information between $i1$ and $i2$.

        The return value has the form [(n0, s0), (n1, s1), ...], where
        n is the number of characters formated with style s.
        """
        return []

    def set_styles(self, i, styles):
        """Sets the styles from position $i$ on according to the list $styles$


        Styles is a list of the form [(n1, s1), (n2, s2), ... ], where
        each s is a style and n is a length in index positions. The
        combination get_styles / set_styles is only used for undoing
        the set_properties-operation.
        """
        pass

    def set_properties(self, i1, i2, properties):
        """Sets style-properties for all texels in the range $i1$ to $i2$.

        The value $properties$ is a dic, e.g.  {'fontsize':16,
        'bgcolor':'red'}.
        """
        pass

    @staticmethod
    def create_group(l):
        return Group(l)



class Group(Texel, treebase.Group):
    """A class which can hold a list (childs) of other texels."""

    def __getstate__(self):
        state = self.__dict__.copy()
        try:
            del state['weights']
        except KeyError:
            pass
        return state

    def __setstate__(self, state):
        self.__dict__ = state
        self.compute_weights()

    def get_text(self):
        return u''.join(texel.get_text() for texel in self.childs)

    def get_style(self, i):
        for texel in self.childs:
            n = len(texel)
            if n > i:
                return texel.get_style(i)
            i -= n

    def get_styles(self, i1, i2):
        styles = []
        for texel in self.childs:
            n = len(texel)
            if i1 < n and i2 >= 0:
                styles = listjoin(styles, texel.get_styles(i1, i2), _style_add)
            i1 -= n
            i2 -= n
        return styles

    def set_styles(self, i, styles):
        data = []
        for texel in self.childs:
            # XXX should be optimized
            data.append(texel.set_styles(i, styles))
            i -= len(texel)
        assert calc_length(data) == len(self)
        g = self.create_group(homogenize(data))
        # merge childs if possible
        j = 0
        for texel in data:
            j += len(texel)
            g = heal(g, j)
        return g

    def set_properties(self, i1, i2, properties):
        assert i2 >= i1
        r = []
        i = 0
        for texel in self.childs:
            n = len(texel)
            if i1 < n and i2 > 0:
                r += texel.set_properties(i1, i2, properties)
            else:
                r.append(texel)
            i1 -= n
            i2 -= n
        try:
            assert is_homogeneous(r)
        except:
            print "before:"
            self.dump()
            print "after:"
            for i, x in enumerate(r):
                print "item", i
                x.dump()
        return groups(r)
G = Group


class Glyph(Texel, treebase.Element):
    """ Baseclass for texels representing a single character (examples:
        newline, tabulator)"""
    style = defaultstyle
    weights = (0, 1, 0)

    def __init__(self, style=None):
        if style is not None:
            self.style = style

    def get_style(self, i):
        return self.style

    def get_text(self):
        return ' '

    def get_styles(self, i1, i2):
        if i2 > 0 >= i1:
            return [(1, self.style)]
        return []

    def set_styles(self, i, styles):
        styles = styles[:]
        while styles:
            l, style = styles[0]
            del styles[0]
            if i <= 0 and i+l > 0:
                clone = shallow_copy(self)
                clone.style = style
                return clone
            i += l
        return self

    def set_properties(self, i1, i2, properties):
        if i1 <= 0 and i2 > 0:
            clone = shallow_copy(self)
            clone.style = updated_style(self.style, properties)
            return [clone]
        return [self]

    def replace_child(self, i1, i2, stuff):
        if i1 == i2 == 0:
            return join(stuff, [self])
        elif i1 == i2 == 1:
            return join([self], stuff)
        raise IndexError, (i1, i2)

    def takeout(self, i1, i2):
        if i1 == i2:
            return [self], []
        return [], [self]



class Characters(Texel, treebase.Element):
    """Texel holding a string of unicode characters."""
    def __init__(self, text, style=defaultstyle):
        unicode(text) # check proper encoding
        self.text = text
        self.style = style
        self.compute_weights()

    def compute_weights(self):
        self.weights = list(self.weights)
        self.weights[1] = len(self.text)

    def __str__(self):
        return "C(%s)" % repr(self.text)

    def __repr__(self):
        return "C(%s, %s)" % (repr(self.text), repr(self.style))

    def __getstate__(self):
        state = self.__dict__.copy()
        del state['weights']
        return state

    def __setstate__(self, state):        
        self.__dict__ = state
        self.compute_weights()

    def get_text(self):
        return self.text

    def get_style(self, i):
        return self.style

    def get_styles(self, i1, i2):
        i1 = max(0, i1)
        i2 = min(len(self), i2)
        if i2 > i1:
            return [(i2-i1, self.style)]
        return []

    def set_styles(self, i, styles):
        text = self.text
        r = []
        while text:
            if not styles:
                texel = self.__class__(text, self.style)
                r.append(texel)
                break
            l, style = styles[0]
            n = len(text)
            if i > n:
                # entry is right of text
                texel = self.__class__(text, self.style)
                r.append(texel)
                break
            elif i+l <= 0:
                # entry is left of text
                i += l
                styles = styles[1:]
                continue
            if i < 0:
                l += i
                i = 0
            assert i >= 0
            if i > 0:
                texel = self.__class__(text[:i], self.style)
                r.append(texel)
                text = text[i:]
                i = 0
            assert l > 0
            assert i == 0
            m = min(l, n)
            texel = self.__class__(text[:m], style)
            r.append(texel)
            if l >= n:
                break
            text = text[m:]
            styles = [(l-m, style)]+list(styles[1:])

        assert calc_length(r) == len(self)
        return self.create_group(r)

    def set_properties(self, i1, i2, properties):
        if i1 == i2:
            return [self]
        style = updated_style(self.style, properties)
        if style is self.style:
            return [self]
        i1 = max(0, i1)
        i2 = min(len(self), i2)
        assert i2 >= i1
        data = self.text
        Characters = self.__class__
        l = Characters(data[:i1], self.style)
        c = Characters(data[i1:i2], style)
        r = Characters(data[i2:], self.style)
        return [x for x in (l, c, r) if len(x)]

    def replace_child(self, i1, i2, stuff):
        Characters = self.__class__
        r1 = Characters(self.text[:i1], self.style)
        r2 = Characters(self.text[i2:], self.style)
        return join(*filter(calc_length, [[r1], stuff, [r2]]))

    def takeout(self, i1, i2):
        if not 0 <= i1 <= i2 <= len(self):
            raise IndexError((i1, i2))
        r1 = self.text[:i1]
        r2 = self.text[i2:]
        r3 = self.text[i1:i2]
        s = self.style
        Characters = self.__class__
        return [Characters(r1+r2, s)], [Characters(r3, s)]

    def can_merge(self, other):
        if isinstance(other, Characters) and self.style is other.style:
            return True

    def merge(self, other):
        assert self.can_merge(other)
        return self.__class__(self.text+other.text, self.style)

    def dump(self, i=0):
        print (" "*i)+str(self), id(self.style)

C = Characters
NULL_TEXEL = Characters('')
SPACE = Characters(' ')



class NewLine(Glyph):
    weights = (0, 1, 1)
    is_endmark = False
    def __repr__(self):
        return 'NL'

    def __str__(self):
        return 'NL'

    def get_text(self):
        return u'\n'

NL = NewLine()
ENDMARK = NewLine()
ENDMARK.is_endmark = True


class Tabulator(Glyph):
    def get_text(self):
        return u'\t'

TAB = Tabulator()




def grouped(stuff):
    # A modified version of treebase.grouped() which can also handle
    # empty lists.
    if not stuff:
        return Group([])
    return _grouped(stuff)





text1 = "0123456789"
text2 = "abcdefghijklmnopqrstuvwxyz"
text3 = "01\n345\n\n89012\n45678\n"

def check(texel):
    """Test wether $texel$ behaves properly."""
    assert isinstance(texel, Texel)

    # pickle/restore must be possible
    import cPickle
    s = cPickle.dumps(texel)
    clone = cPickle.loads(s)
    assert depth(clone) == depth(texel)
    assert len(clone) == len(texel)
    assert clone.weights == texel.weights

    # insert must be possible in alle positions
    x = Characters('X')
    for i in range(len(texel)+1):
        simple_insert(texel, i, [x])

    # inserting via "replace_child" must be possible at first and last and
    # for all child positions
    for i in [0, len(texel)]+[i1 for (i1, i2, c) in texel.iter_childs()]:
        texel.replace_child(i, i, [x])
        texel.replace_child(i, i, [x])

    # removing via "replace_child" must be possible for all child intervals
    for i1, i2, child in texel.iter_childs():
        texel.replace_child(i1, i2, [])

    # takeout must be possible for all positions in a child
    for i1, i2, child in texel.iter_childs():
        for i in range(i2-i1):
            texel.takeout(i1+i, i1+i+1)

    # takeout must be possible across child boundaries
    l = []
    for i1, i2, child in texel.iter_childs():
        if l:
            if l[-1][1] == i1:
                j1, j2 = l.pop()
                l.append((j1, i2))
                continue
        l.append([i1, i2])
    for i1, i2 in l:
        for i in range(i2-i1-1):
            texel.takeout(i1+i, i1+i+2)

    # takeout must raise an IndexError if it is tried to remove an
    # empty position
    l = [] # empty positions
    last = None
    for i1, i2, child in texel.iter_childs():
        if last is not None:
            if i1 > last:
                assert i1-last == 1 # empty positions can't be neighbours
                l.append(last)
        last = i2
    for i in l:
        ok = False
        try:
            texel.takeout(i, i+1)
        except IndexError:
            ok = True
        assert ok

    # Style must be defined and writeable for all n positions
    s1 = create_style(fontsize = -1)
    s2 = create_style(fontsize = -2)
    for i in range(len(texel)):
        texel.get_style(i)
        tmp = texel.set_styles(i, [(1, s1)])
        try:
            assert tmp.get_style(i) == s1
        except:
            print i, tmp.get_text()[i:i+1]
            tmp.dump()
            raise
        tmp = texel.set_properties(i, i+1, {'fontsize':-2})
        assert G(tmp).get_style(i) == s2
    return True


def check_split(texel):
    for i in range(len(texel)+1):
        texel.takeout(0, i)
        texel.takeout(i, len(texel))
    return True

if debug:
     #enable contract checking
     import contract
     contract.checkmod(__name__)


def test_00():
    "Characters"
    s = Characters(text1)
    assert check_split(s)
    assert check(s)


def test_01():
    "Group"
    s = Characters(text1)
    s1 = Characters(text1)
    s2 = Characters(text1)

    s1 = Characters(text1)
    s2 = Characters(text1, style=create_style(fontsize=4))

    # we need to set nmax to avoid that the efficiency tests to
    # complain about our constructed groups.
    treebase.set_nmax(3)

    c = Group([s1, s2])
    assert check(c)
    assert check_split(c)

    c1 = Group([c, c])
    assert check(c1)
    check_split(c1)

    c1 = Group([c, Group([Characters("X"), Characters("Y")]), c])
    assert is_root_efficient(c1)
    assert check(c1)
    assert check_split(c1)

    c1 = Group([c]*20)



def test_02():
    "insert and heal"

    t1 = Characters(text1)
    t2 = Characters(text2)

    # check that texts with same styling are merged
    for i in range(len(t1)+1):
        t = heal(grouped(insert(t1, i, [t2])), i, i+len(t2))
        if not isinstance(t, Characters):
            t.dump()
        assert isinstance(t, Characters)

    g = Group([t2])
    # check that group is removed by heal
    for i in range(len(t1)+1):
        t = grouped(insert(t1, i, [g]))
        t = heal(t, i, i+len(g))
        assert isinstance(t, Characters)


def test_05():
    "insert"
    c = Characters(text1)
    g = Group([c])
    n = NL
    items = [c, g, n]
    for item in items:
        for other in items:
            n = len(item)
            no = len(other)
            item_text = item.get_text()
            other_text = other.get_text()
            for i in range(len(item)+1):
                r = grouped(insert(item, i, [other]))
                assert len(r) == n+no
                assert r.get_text() == item_text[:i]+other_text+item_text[i:]


def test_06():
    "heal"
    C = Characters
    G = Group
    t = G([C("01234"), C(""), C("56789")])
    assert str(heal(t, 5)) == "C('0123456789')"

    t = G([G([C("01234")]), G([ C(""), C("56789")])])
    assert str(heal(t, 5)) == "C('0123456789')"

    t = G([G([C("01234"), C(""), C("")]), G([ C(""), C("56789")])])
    assert str(heal(t, 5)) == "C('0123456789')"


def test_07():
    "set properties"
    texel = Characters('0123')
    from random import randrange, choice

    defaultstyle.clear()
    defaultstyle['s'] = 10

    n = len(texel)
    for j in range(2000):
        i1 = randrange(n)
        i2 = randrange(n)
        i1, i2 = sorted([i1, i2])
        size = choice([10, 14])
        new = heal(grouped(texel.set_properties(i1, i2, {'s':size})), i1, i2)
        assert not "C(u'')" in str(texel)
        assert not "C(u'')" in str(new)
        texel = new
    #texel.dump()


def test_08():
    "get/set styles"

    s0 = defaultstyle
    s1 = create_style(bgcolor='red')
    s2 = create_style(bgcolor='yellow')

    def styles2str(styles):
        r = []
        for l, style in styles:
            i = 'x'
            if style == s0:
                i = "0"
            elif style == s1:
                i = "1"
            elif style == s2:
                i = "2"
            r.append(i*l)
        return ''.join(r)

    t = Characters(text1)
    assert t.get_styles(0, len(t)) == [(len(t), s0)]
    t = t.set_styles(6, [(1, s2)])
    #print styles2str(t.get_styles(0, len(t)))
    assert styles2str(t.get_styles(0, len(t))) == '0000002000'

    t = Characters(text1)
    t = grouped(t.set_properties(3, 5, {'bgcolor':'red'}))
    assert styles2str(t.get_styles(0, len(t))) == '0001100000'
    assert styles2str(t.get_styles(1, len(t))) == '001100000'
    assert styles2str(t.get_styles(4, len(t))) == '100000'

    t = t.set_styles(6, [(1, s2)])
    assert styles2str(t.get_styles(0, len(t))) == '0001102000'

    # override styling
    n = len(t)
    styles = t.get_styles(1, len(t))
    t = t.set_styles(5, [(len(t)-5, s0)])
    assert styles2str(t.get_styles(0, len(t))) == '0001100000'

    # restore styling
    t = t.set_styles(1, styles)
    styles = t.get_styles(1, len(t))
    assert styles2str(t.get_styles(0, len(t))) == '0001102000'

    # strings with same styling should be merged
    t = t.set_styles(3, [(10, s0)])
    assert isinstance(t, Characters) # assert that we have only one texel

    # merging styles
    styles = Group([Characters(text1), Characters(text1)]).\
        get_styles(0, 2*len(text1))
    assert len(styles) == 1
    assert styles[0][0] == 2*len(text1)

def test_09():
    "NewLine"
    texel = NewLine()
    assert check(texel)

    c = Group([C('0123456789'), NewLine(), C('BCD')])
    assert check(c)


def test_10():
    "optimized growth"

    def maxn(texel):
        if isinstance(texel, Group):
            n = len(texel.childs)
            return max(n, max([maxn(g) for g in texel.childs]))
        return 1

    def check_depth(texel):
        # assert uniform_depth(texel)
        if not isinstance(texel, Group):
            return True
        if not len(texel.childs):
            return True
        d = depth(texel)
        for child in texel.childs:
            if not depth(child) == d-1:
                texel.dump()
            assert depth(child) == d-1
        for child in texel.childs:
            assert check_depth(child)
        return True

    def check_nchilds(texel, is_root=True):
        if isinstance(texel, Group):
            if not is_root:
                assert len(texel.childs) >= treebase.nmax/2
            assert len(texel.childs) <= treebase.nmax
            for child in texel.childs:
                assert check_nchilds(child, False)
        return True

    for n in (3, 4, 5, 6):
        treebase.set_nmax(n)
        t =  Characters("X")
        for i in range(30):
            # create new styles to suppress merging of strings
            tt = Characters("%i " % i, style=create_style(fontsize=i))
            old = t
            t = grouped(insert(t, 0, [tt]))
            assert check_depth(t)
            assert check_nchilds(t)
    #t.dump()

def test_10():
    "optimized growth"

    for n in (3, 4, 5, 6):
        #print "nmax = ", n
        treebase.set_nmax(n)
        t =  Characters("X")
        for i in range(30):
            # create new styles to suppress merging of strings
            tt = Characters("%i " % i, style=create_style(fontsize=i))
            old = t
            t = grouped(insert(t, 0, [tt]))
            try:
                assert is_root_efficient(t)
            except:
                print "t is not efficient:"
                t.dump()
                raise





# -*- coding: latin-1 -*-

from . import treebase
from . import texeltree
from .texeltree import Texel, NULL_TEXEL, G, grouped, defaultstyle, \
      updated_style, _style_add, SPACE
from .treebase import join, simple_insert
from .listtools import calc_length, listjoin


class Container(Texel, treebase.Element):
    has_childs = True
    def __init__(self, style=defaultstyle):
        self.style = style
        w_list = zip(*[child.weights for i1, i2, child in self.iter_extended()])
        self.weights = [f(l) for (l, f) in zip(w_list, self.functions)]
        self.weights[0] = 0 # Containers have depth 0!

    def get_content(self):
        raise NotImplemented()

    def get_kwds(self):
        return {'style' : self.style}

    def is_emptypos(self, i):
        for j1, j2, child in self.iter_childs():
            if i == j1-1:
                return True
        if i == len(self):
            return True

    def iter_extended(self):
        empties = self.get_empties()
        yield 0, 1, empties[0]
        i1 = 1
        for child, empty in zip(self.get_content(), empties[1:]):
            i2 = i1+len(child)
            yield i1, i2, child
            yield i2, i2+1, empty
            i1 = i2+1

    ### default implementations
    def get_emptychars(self):
        # default: treat empty positions as spaces
        return u' ' * (len(self.get_content())+1)

    def get_empties(self):
        # Can be overridden
        return (SPACE,) * (len(self.get_content())+1)

    def get_child(self, i):
        for j1, j2, child in self.iter_childs():
            if j1 <= i <= j2:
                return j1, j2, child
        raise IndexError(i)

    ### texel interface
    def get_style(self, i):
        for j1, j2, child in self.iter_childs():
            if j1 <= i < j2:
                return child.get_style(i-j1)
        return self.style

    def get_text(self):
        emptychars = self.get_emptychars()
        content = self.get_content()
        assert len(emptychars) == len(content)+1
        t = emptychars[0]
        for i, child in enumerate(content):
            t += child.get_text()+emptychars[i+1]
        return t

    def get_styles(self, i1, i2):
        styles = []
        if i1 < 1 and i2 > 0:
            styles.append((1, self.style))
        i1 -= 1
        i2 -= 1
        for texel in self.get_content():
            n = len(texel)
            if i1 < n and i2 > 0:
                #print i1, i2, texel.get_styles(i1, i2)
                styles = listjoin(styles, texel.get_styles(i1, i2),
                                  _style_add)
                #styles.append(texel.get_styles(i1, i2))
            i1 -= n
            i2 -= n
            if i1 < 1 and i2 > 0:
                styles = listjoin(styles, [(1, self.style)],
                                  _style_add)
                #styles.append((1, self.style))
            i1 -= 1
            i2 -= 1
        return styles

    def set_styles(self, i, styles):
        content = list(self.get_content())
        j = -1
        for j1, j2, child in self.iter_childs():
            j += 1
            if  i<j2:
                content[j] = child.set_styles(i-j1, styles)
        empties = set([0]) # empty indices
        for j1, j2, child in self.iter_childs():
            empties.add(j1-1)
            empties.add(j2)
        empties = list(sorted(empties))
        j1 = i
        style = self.style
        for j, _style in styles:
            j2 = j1+j
            for e in empties:
                if j1 <= e <= j2:
                    style = _style
        kwds = self.get_kwds()
        kwds.update(style=style)
        return self.__class__(*content, **kwds)

    def set_properties(self, i1, i2, properties):
        content = list(self.get_content())
        style = self.style
        for j1, j2, child in self._iter_extended():
            if  i1 < j2 and j1 < i2: # test of overlap
                if child is None:
                    style = updated_style(style, properties)
                else:
                    new = grouped(child.set_properties(i1-j1, i2-j1,
                                                   properties))
                    i = content.index(child)
                    content[i] = new
        kwds = self.get_kwds()
        kwds['style'] = style
        return [self.__class__(*content, **kwds)]

    ### Element interface
    def __len__(self):
        childs = self.get_content()
        return len(childs)+1+calc_length(childs)

    def iter_childs(self):
        i1 = 1
        for child in self.get_content():
            i2 = i1+len(child)
            yield i1, i2, child
            i1 = i2+1

    def _iter_extended(self):
        # Helper
        yield 0, 1, None
        i1 = 1
        for child in self.get_content():
            i2 = i1+len(child)
            yield i1, i2, child
            yield i2, i2+1, None
            i1 = i2+1

    def replace_child(self, i1, i2, stuff):
        content = list(self.get_content())
        if i1 == i2: # insert
            if i1 == 0:
                 return join(stuff, [self])
            elif i1 == len(self):
                return join([self], stuff)
            j1, j2, child = self.get_child(i1)
            if i1 == j1:
                new = grouped(join(stuff, [child]))
            else:
                if not i1 == j2:
                    raise IndexError((i1, i2))
                new = grouped(join([child], stuff))
            i = content.index(child)
            content[i] = new
        else: #replace_child
           j1, j2, child = self.get_child(i1)
           if not (j1 == i1 and j2 == i2):
               # i1..i2 must be the interval occupied by one child.
               raise IndexError((i1, i2))
           i = content.index(child)
           content[i] = grouped(stuff)
        return [self.__class__(*content, **self.get_kwds())]

    def takeout(self, i1, i2):
        assert i1 >= 0
        assert i2 <= len(self)
        if i1 == i2:
            return [self], []
        if i1 <= 0 and i2 >= len(self):
            return [], [self]
        for k, (j1, j2, child) in enumerate(self.iter_childs()):
            if  i1 < j2 and j1 < i2: # test of overlap
                if not (j1 <= i1 and i2 <= j2):
                    raise IndexError((i1, i2))
                childs = list(self.get_content())
                tmp, kern = child.takeout(
                    max(0, i1-j1), min(len(self), i2-j1))
                childs[k] = grouped(tmp)
                return [self.__class__(*childs, **self.get_kwds())], \
                       kern
        raise IndexError((i1, i2))


def iter_extended(texel):
    if isinstance(texel, Container):
        for i1, i2, child in texel.iter_extended():
            yield i1, i2, child
    else:
        for i1, i2, child in texel.iter_childs():
            yield i1, i2, child


class Fraction(Container):
    # a simple demo
    def __init__(self, denom, nom, style=defaultstyle):
        self.denom = denom
        self.nom = nom
        Container.__init__(self, style)

    def get_content(self):
        return self.denom, self.nom

    def get_emptychars(self):
        return '(;)'


def test_00():
    "get_styles/set_styles"

    from .texeltree import Characters
    frac = Fraction(Characters("denominator"), Characters("nominator"))
    n = len(frac)
    assert frac.get_styles(0, n) == [(n, defaultstyle)]
    assert frac.get_styles(1, n) == [(n-1, defaultstyle)]
    assert frac.get_styles(2, n) == [(n-2, defaultstyle)]
    assert frac.get_styles(2, n-1) == [(n-3, defaultstyle)]
    assert frac.get_styles(0, 1) == [(1, defaultstyle)]

    tmp = frac.set_styles(0, [(1, None)])
    assert tmp.get_styles(0, 1) == [(1, None)]
    assert tmp.get_styles(0, 2) == [(1, None), (1, defaultstyle)]

def test_01():
    "get_text"
    from .texeltree import Characters
    frac = Fraction(Characters("denominator"), Characters("nominator"))
    assert frac.get_text() == '(denominator;nominator)'

def test_02():
    "insert"
    from .texeltree import Characters
    from .treebase import insert
    frac = Fraction(Characters("denominator"), Characters("nominator"))
    elem = frac
    text = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMN"
    for i in range(len(text)):
        #print i
        tmp = insert(elem, 17, [Characters(text[i])])
        assert calc_length(tmp) == len(elem)+1
        elem = grouped(tmp)

    from .treebase import dump
    #dump(elem)

def test_010():
    from .texeltree import Characters, check
    frac = Fraction(Characters("denominator"), Characters("nominator"))
    assert check(frac)
    assert frac.get_text() == '(denominator;nominator)'
    tmp = frac.replace_child(1, 12, [Characters('x')])[0]
    assert tmp.get_text() == '(x;nominator)'
    try:
        frac.replace_child(1, 13, [Characters('x')])
        assert False
    except IndexError:
        pass

    rest, kern = frac.takeout(5, 11)
    assert G(rest).get_text() == '(denor;nominator)'
    assert G(kern).get_text() == 'minato'

    rest, kern = frac.takeout(13, 14)
    assert isinstance(rest[0], Fraction)
    assert G(rest).get_text() == '(denominator;ominator)'
    assert G(kern).get_text() == 'n'
    assert len(G(rest)) == len(frac)-1
    assert len(G(kern)) == 1

    for i in range(len(frac)+1):
        if frac.is_emptypos(i):
            continue
        simple_insert(frac, i, [Characters('x')])

def test_011():
    "heal"
    from .texeltree import Group, Characters, TAB, heal
    frac = Fraction(Characters("denominator"), Characters("nominator"))
    assert frac.get_text()[13:14] == 'n'
    #(denominator;nominator)
    tree = G(simple_insert(frac, 13, [TAB]))
    tree.dump()
    heal(tree, 13, 14).dump()
    
    tree = G(simple_insert(frac, 13, [Characters('X')]))
    tree.dump()
    t = heal(tree, 13) # no merge possible
    assert isinstance(t.nom, Group)
    t = heal(tree, 14) # merge possible
    assert isinstance(t.nom, Characters)



# -*- coding: latin-1 -*-

# Simple layout for text editors
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
# The layout can be characterized by three different layers of boxes:

# paragraphs, lines and boxes. The top layer is the paragraph
# layer. It consists of a tree formed by ParagraphStacks and
# Paragraphs. Each paragraph has a tree of lines. Each line has a list
# of boxes.
#
# All paragraphs are stacked on top of each other. Each line has
# exactly the same width.
#
# When the model changes, we identify the corresponding paragraph
# objects, rebuild them and replace them.


from .boxes import HBox, VBox, VGroup, TextBox, EmptyTextBox, NewlineBox, EndBox, \
                   check_box, Box, tree_depth, replace_boxes, Row
from .boxes import grouped as _grouped
from ..textmodel.texeltree import NewLine, Characters, defaultstyle
from ..textmodel.treebase import groups
from ..textmodel import listtools, treebase

from .testdevice import TESTDEVICE
from .linewrap import simple_linewrap
from .rect import Rect
from .builder import BuilderBase
from .builder import Factory as _Factory



class Paragraph(VBox):
    # A paragraph consists of one or several rows, which are stacked
    # on top of each other. It is assumed, that the text consists of
    # one single page with a constant left margin and uniform row
    # width. These assumptions correspond to the typical behaviour in
    # a text editor. If there are different requirements, e.g. in a
    # word processor, Paragraph could be redefined.
    #
    # The number of paragraphs can be very long. Therefore we group
    # paragraph into VGroups. This makes the GUI considerably faster.

    def create_group(self, l):
        return VGroup(l, device=self.device)




def get_envelope(tree, i0, i):
    # Returns i1, i2 where i1<=i<=i2 and there is one or none
    # paragraph in i1 .. i2.
    assert i>=0 
    assert i<=len(tree)
    if i == 0 or i == len(tree):
        return i0+i, i0+i
    if isinstance(tree, Paragraph):
        return i0, i0+len(tree)
    else:
        for i1, i2, child in tree.iter_childs():
            if i1<=i<=i2:
                return get_envelope(child, i0+i1, i-i1)
    raise IndexError(i)



def create_paragraphs(textboxes, maxw=0, Paragraph=Paragraph, \
                      device=TESTDEVICE):
    try:
        if len(textboxes):        
            assert isinstance(textboxes[-1], NewlineBox) or \
                isinstance(textboxes[-1], EndBox)
    except:
        i1 = 0
        for box in textboxes:
            i2 = i1+len(box)
            print i1, i2, repr(box)[:50]
            i1 = i2
        raise
    r = []
    l = []
    for box in textboxes:
        assert isinstance(box, Box)
        l.append(box)
        if isinstance(box, NewlineBox) or isinstance(box, EndBox):
            if maxw>0:
                rows = simple_linewrap(l, maxw, tabstops=(), device=device)
            else:
                rows = [Row(l, device)]
            r.append(Paragraph(rows, device))
            l = []
    #print l
    assert not l # There is always one final box which is either a
                 # NewLine or an EndBox. Therefore there is no rest in
                 # l.

    assert listtools.calc_length(r) == listtools.calc_length(textboxes)
    while len(r)>treebase.nmax:
        r = groups(r)
    return r


class Factory(_Factory):
    def create_paragraphs(self, texel, i1, i2):
        # creates a list of paragraphs
        textboxes = self.create_boxes(texel, i1, i2)
        assert len(textboxes)
        assert isinstance(textboxes[-1], EndBox) or isinstance(textboxes[-1], \
                                                               NewlineBox)
        return create_paragraphs(
            textboxes, self._maxw, 
            Paragraph = self.Paragraph,
            device = self.device,
        )



class Builder(BuilderBase, Factory):
    Paragraph = Paragraph

    def __init__(self, model, device=TESTDEVICE, maxw=0):
        self.model = model
        self._maxw = maxw
        Factory.__init__(self, device)

    def grouped(self, stuff):
        # We need to create a new version of grouped, again. This time
        # it must be aware of VGroup. Since it must be able to pass
        # device, this time it is a method and not a function.
        if not stuff:
            r = VGroup([], device=self.device)
            return r
        return treebase.grouped(stuff)

    def replace_paragraphs(self, i1, i2, stuff):
        # Helper: replaces all paragraphs between $i1$ and $i2$ by
        # $stuff$, where $stuff$ is a list of paragraphs.
        self._layout = self.grouped(replace_boxes(self._layout, i1, i2, stuff))

    def get_envelope(self, i1, i2):
        # Helper: adjust $i1$ and $i2$ to the beginning / end of a paragraph 
        k1, k2 = get_envelope(self._layout, 0, i1)
        j1, j2 = get_envelope(self._layout, 0, i2)
        return k1, j2

    def extended_texel(self):
        return self.model.get_xtexel()
        
    def set_maxw(self, maxw):
        if maxw != self._maxw:
            self._maxw = maxw
            self.rebuild()
        
    ### Builder-Protocol
    def rebuild(self):
        texel = self.extended_texel()
        l = self.create_paragraphs(texel, 0, len(texel))
        self._layout = self.grouped(l)
        assert isinstance(self._layout, Box)

    ### Signal handlers
    def properties_changed(self, i1, i2):
        #print "properties changed", i1, i2
        j1, j2 = self.get_envelope(i1, i2)
        texel = self.extended_texel()
        new = self.create_paragraphs(texel, j1, j2)
        self.replace_paragraphs(j1, j2, new)

    def inserted(self, i, n):
        j1, j2 = self.get_envelope(i, i+1) # +1 is needed when we
                                            # insert between two
                                            # paragraphs
        texel = self.extended_texel()
        new = self.create_paragraphs(texel, j1, j2+n)
        self.replace_paragraphs(j1, j2, new)

    def removed(self, i, n):
        #print "removed", i, n
        i1 = i
        i2 = i+n
        if i2<len(self._layout):
            # Removing the NL at the paragraph end meens, that the
            # paragraph sould be merged with the next paragraph. We
            # therefore have to extend the interval so that both
            # paragraphs are rebuild.
            i2 = i2+1
        j1, j2 = self.get_envelope(i1, i2)
        texel = self.extended_texel()
        new = self.create_paragraphs(texel, j1, j2-n)
        self.replace_paragraphs(j1, j2, new)



def _create_testobjects(s):
    from ..textmodel.textmodel import TextModel
    texel = TextModel(s).texel    
    box = TextBox(s)
    return box, texel


def test_00():
    "Paragraph"
    box1, tmp = _create_testobjects("0123")
    box2, tmp = _create_testobjects("5678")
    tmp, texel = _create_testobjects("0123\n5678\n")
    box = Paragraph([
        Row([box1, NewlineBox()]), 
        Row([box2, NewlineBox()]),
        Row([EmptyTextBox()])
    ])
    assert check_box(box, texel)
    #box.dump_boxes(0, 0, 0)
    assert (box.height, box.width, box.depth) == (3, 4, 0)

    assert str(box.get_info(0, 0, 0)) == "(TB('0123'), 0, 0, 0)"
    assert str(box.get_info(1, 0, 0)) == "(TB('0123'), 1, 1, 0)"
    assert str(box.get_info(2, 0, 0)) == "(TB('0123'), 2, 2, 0)"
    assert str(box.get_info(3, 0, 0)) == "(TB('0123'), 3, 3, 0)"
    assert str(box.get_info(4, 0, 0)) == "(NL, 0, 4, 0)"
    assert str(box.get_info(5, 0, 0)) == "(TB('5678'), 0, 0, 1)"
    assert str(box.get_info(6, 0, 0)) == "(TB('5678'), 1, 1, 1)"
    assert str(box.get_info(7, 0, 0)) == "(TB('5678'), 2, 2, 1)"
    assert str(box.get_info(8, 0, 0)) == "(TB('5678'), 3, 3, 1)"
    assert str(box.get_info(9, 0, 0)) == "(NL, 0, 4, 1)"
    assert str(box.get_info(10, 0, 0)) == "(ETB, 0, 0, 2)"


def test_01():
    "Row"
    box1, tmp = _create_testobjects("0123")
    box2, tmp = _create_testobjects("5678")
    tmp, texel1 = _create_testobjects("0123\n")
    tmp, texel2 = _create_testobjects("5678\n")
    tmp, texel = _create_testobjects("0123\n5678\n")
    p1 = Paragraph([
        Row([box1, NewlineBox()]), 
    ])
    p2 = Paragraph([
        Row([box2, NewlineBox()]), 
        Row([EmptyTextBox()])
    ])
    assert check_box(p1, texel1)
    assert check_box(p2, texel2)
    box = _grouped([p1, p2])
    assert isinstance(box, VBox)
    assert check_box(box, texel)

    assert str(box.get_info(0, 0, 0)) == "(TB('0123'), 0, 0, 0)"
    assert str(box.get_info(1, 0, 0)) == "(TB('0123'), 1, 1, 0)"
    assert str(box.get_info(2, 0, 0)) == "(TB('0123'), 2, 2, 0)"
    assert str(box.get_info(3, 0, 0)) == "(TB('0123'), 3, 3, 0)"
    assert str(box.get_info(4, 0, 0)) == "(NL, 0, 4, 0)"
    assert str(box.get_info(5, 0, 0)) == "(TB('5678'), 0, 0, 1)"
    assert str(box.get_info(6, 0, 0)) == "(TB('5678'), 1, 1, 1)"
    assert str(box.get_info(7, 0, 0)) == "(TB('5678'), 2, 2, 1)"
    assert str(box.get_info(8, 0, 0)) == "(TB('5678'), 3, 3, 1)"
    assert str(box.get_info(9, 0, 0)) == "(NL, 0, 4, 1)"
    assert str(box.get_info(10, 0, 0)) == "(ETB, 0, 0, 2)"

    #box.dump_boxes(0, 0, 0)

    box2 = _grouped(replace_boxes(box, 5, 10, []))
    box2.dump_boxes(0, 0, 0)
    assert len(box2) == 5

    xbox, tmp = _create_testobjects("X")
    p = Paragraph([xbox])
    box2 = _grouped(replace_boxes(box, 5, 10, [p]))
    #box2.dump_boxes(0, 0, 0)
    assert len(box2) == 6
    from ..textmodel import treebase
    treebase.nmax = 5
    box2 = Paragraph([
        Row([EmptyTextBox()])
    ])

    l = []
    for i in range(15):
        ibox, tmp = _create_testobjects(str(i))
        r = Row([ibox, NewlineBox()])
        l.append(Paragraph([r]))
    for x in l:
        assert isinstance(x, Paragraph)
    box2 = _grouped(replace_boxes(box, 5, 10, l))
    #box2.dump_boxes(0, 0, 0) # check that the tree is balenced    
    return box2


def test_02():
    "get_envelope"
    box = test_01()
    #box.dump_boxes(0, 0, 0)
    assert get_envelope(box, 0, 0) == (0, 0)
    assert get_envelope(box, 0, 1) == (0, 5)
    assert get_envelope(box, 0, 5) == (5, 5)
    assert get_envelope(box, 0, 6) == (5, 7)

def _mk_pars(text):
    # for testing
    l = []
    for line in text.split('\n'):
        for word in line.split():
            l.append(TextBox(word))
        l.append(NewlineBox(defaultstyle))
    return create_paragraphs(l)

def test_02a():
    "create_paragraphs"
    paragraphs = _mk_pars("word1 word2 word3")
    assert str(paragraphs) == "[Paragraph[Row[TB('word1'), TB('word2'), TB('word3'), NL]]]"
    paragraphs = _mk_pars("word1\nword2\nword3")
    assert str(paragraphs) == "[Paragraph[Row[TB('word1'), NL]], Paragraph[Row[TB('word2')," \
                              " NL]], Paragraph[Row[TB('word3'), NL]]]"

def xxtest_02b():
    "insert_paragraphs"
    box = test_01()
    paragraphs = _mk_pars("xx yy zz")
    box = _grouped(insert_paragraphs(box, 5, paragraphs))
    box.dump_boxes(0, 0, 0)
    assert get_envelope(box, 0, 1) == (0, 5)
    print get_envelope(box, 0, 6)
    assert get_envelope(box, 0, 6) == (5, 12)

def test_03():
    t1 = TextBox("0123456789")
    t2 = TextBox("0123456789")
    p1 = Paragraph([Row([t1, NewlineBox(defaultstyle)])])
    row = p1.childs[0]
    assert p1.height == 1
    assert p1.width == 10
    assert len(p1) == 11
    p2 = VBox([Row([t2])])
    assert p2.height == 1
    s = VBox([p1, p2])
    assert s.height == 2    


def test_04():
    # Problem: get_rect always returns 0, 0
    t1 = TextBox("0123456789")
    t2 = TextBox("0123456789")
    p1 = Paragraph([Row([t1, t2, NewlineBox(defaultstyle)])])

    assert p1.get_rect(0, 0, 0) == Rect(0, 0.0, 1, 1.0)
    assert p1.get_rect(10, 0, 0) == Rect(10, 0.0, 11, 1.0)

def test_05():
    t1 = TextBox("0123456789")
    t2 = TextBox("0123456789")
    NL = NewlineBox()
    p1 = Paragraph([Row([t1, NL])])
    assert tree_depth(p1) == 0
    tmp = _grouped([p1])
    assert not tmp.is_group
    print tmp
    tmp.dump()
    assert tmp is p1

    p2 = Paragraph([Row([t2, NL])])
    print p2, tree_depth(p2)
    assert tree_depth(p2) == 0
    
    tmp = _grouped([p1, p2])
    tmp.dump()
    assert tmp.is_group
    print tree_depth(tmp)
    assert tree_depth(tmp) == 1
    # ...





# -*- coding: latin-1 -*-



"""treebase.py -- basic hierarchical data structure

This file defines a hierarchical data structure consisting of
elements. A special element is the group element, which can hold child
elements.

Elements have a certain length, a depth and a weights as defined below.

Technically, elements span a tree with groups as branches. Unlike
binary trees, groups can have more than two childs. Ideally groups
have between nmax/2 and nmax childs. If this condition is fulfilled
for all groups in the tree, the tree is said to be efficient.

With nmax = 15, one tree of depth 5 can carry up to 759,375 leave
elements, which is be more than enough for our purpose.

"""

from .listtools import calc_length
from . import listtools

debug = 0 # don't check
#debug = 1 # check
#debug = 2 # also check efficiency

nmax = 15
def set_nmax(i):
    """Sets $nmax$ to the value $i$.

       The value nmax determines the number of childs per group (which
       should be between nmax/2 and nmax for efficient trees).
    """
    global nmax
    nmax = i


def is_group(element):
    try:
        return element.is_group
    except AttributeError:
        return False


def depth(element):
    """Returns the depth of an element.

       The depth can be definied as the number of generations of
       groups. By definition empty groups have depth 0.

       >>> depth(Characters('abc'))
       0

       >>> depth(G(Characters('abc')))
       1

       >>> depth(G()) # empty groups are ignored
       0
    """
    if is_group(element):
        return element.get_depth()
    return 0


def count(element):
    """Returns the number of elements.

       This function is very slow and is only ment for debugging. 
    """
    n = 1
    for i1, i2, child in element.iter_childs():
        n += count(child)
    return n



def is_homogeneous(l):
    """Returns True if the elements in list *l* are homogeneous.

       Elements are homogeneous if they have alle the same depth.
    """
    return maxdepth(l) == mindepth(l)


def is_element_efficient(element):
    """Returns True if $element$ is efficient.

       An element is efficient, if it is not a group or otherwise if
       all of the following criteria are fulfilled:

       1. if each descendant group has between nmax/2 and nmax childs
       2. the depth of all childs is homogeneous

       In an efficient tree all groups (with the exception of the root
       node) must be element_efficient.

       Note: this function is very slow and should only be used for
       debugging.
    """
    if not is_group(element):
        return True
    if not is_homogeneous(element.childs):
        return False
    if len(element.childs) < nmax/2 or len(element.childs) > nmax:
        return False
    if not is_list_efficient(element.childs):
        return False
    return True


def is_list_efficient(l):
    """Returns True if the list of $l$ of elements is efficient.
    """
    if not is_homogeneous(l):
        return False
    for element in l:
        if not is_element_efficient(element):
            return False
    return True


def is_root_efficient(root):
    """Returns True if $root$ is efficient.

       The tree spawned by $root$ is efficient if it is either not a
       group or otherwise if it fulfills all of the following
       conditions:

       1. all childs are efficient
       2. the number of childs is <= nmax

       Note: this function is very slow and should only be used for
       debugging.
    """
    if not is_group(root):
        return True
    if len(root.childs) > nmax:
        return False
    return is_list_efficient(root.childs)


def is_elementlist(l):
    """Returns True if $l$ is a list of Elements.
    """
    if not type(l) in (tuple, list):
        print "not a list or tuple", type(l), l.__class__ # XXX remove this
        return False
    return not False in [isinstance(x, Element) for x in l]


def maxdepth(l):
    """Computes the maximal depth of all elements in list $l$.
    """
    m = [depth(x) for x in l if len(x)]
    if not m:
        return 0
    return max(m)


def mindepth(l):
    """Computes the minimal depth of all elements in list $l$.
    """
    m = [depth(x) for x in l if len(x)]
    if not m:
        return 0
    return min(m)


def strip(element):
    """Removes unnecessary Group-elements from the root.
    """
    n = len(element)
    while is_group(element) and len(element.childs) == 1:
        element = element.childs[0]
    assert n == len(element)
    return element


def out(*args): # XXX remove this
    print repr(args)
    return True


def dump(element, name=''): # XXX remove this
    print "Dump %s:" % name
    element.dump()
    return True

def dump_list(l):
    print "Dumping list (efficient: %s)" % is_list_efficient(l)
    for i, element in enumerate(l):
        print "Dumping element no. %i" % i,
        print "(efficient: %s)" % is_element_efficient(element)
        element.dump()
    return True



class Element:
    """Baseclass for all tree elements.

If element is a container, it must provide the "iter_childs" and
"replace_child" methods. Containers are marked by the flag
"has_childs". Groups are marked by the flag "is_group".
    """
    weights = (0, 0)
    functions = (
        lambda l:max(l)+1, # depth
        sum,               # length
        )

    has_childs = False # if true, childs can be accessed using the
                       # iter_childs method
    is_group = False   # if true, this element behaves as a group
    def get_depth(self):
        """Returns the depth of $self$."""
        return self.weights[0]

    def __len__(self):
        """Returns the length of $self$ in index units."""
        return self.weights[1]

    def iter_childs(self):
        """Iterates over all child elements.

        Usage:
        for i1, i2, child in element.iter_childs():
            ...
        """
        if 0: yield 0, 0, None

    def replace_child(self, i1, i2, stuff):
        """Replace the direct childs between $i1$ and $i2$ by $stuff$.

        Returns a list of elements. The indices $i1$ and $i2$ must lie
        at boundaries. Replace_Child can be used to insert, replace or
        remove child elements.

        element.replace_child(i1, i1, new) -- insert at index i1
        element.replace_child(len(element), len(element), new) -- append
        element.replace_child(i1, i2, []) -- remove everything in i1..i2

        The interval i1..i2 can contain any number of childs (including zero).
        XXX It probably should be renamed in "replace_childs".


        pre:
            is_elementlist(stuff)

        post[]:
            is_elementlist(__return__)
            is_homogeneous(__return__)
            #out(repr(self)[:100])
            #dump(G(__return__))
            calc_length(__return__) == len(self)-(i2-i1)+calc_length(stuff)

        """
        raise NotImplementedError()

    def takeout(self, i1, i2):
        """Takes out all content between $i1$ and $i2$.

        Returns the rest and the cut out piece, i.e.
        G([a, b, c]).takeout(i1, i2) will return G([a, c]), b.

        pre:
            0 <= i1 <= i2 <= len(self)
            is_root_efficient(self)

        post[i1, i2, self]:
            is_elementlist(__return__[0])
            is_elementlist(__return__[1])
            is_homogeneous(__return__[0])
            is_homogeneous(__return__[1])
            calc_length(__return__[0])+i2-i1 == len(self)
            calc_length(__return__[1]) == i2-i1
            is_list_efficient(__return__[0])
            is_list_efficient(__return__[1])
        """
        print self, len(self), i1, i2 # XXX
        raise NotImplementedError()

    def can_merge(self, other):
        """Can element $self$ be merged to other?"""
        return False

    def merge(self, other):
        """Merges $self$ and $other$ to into a single element.

        post[self, other]:
            len(__return__) == len(self)+len(other)
        """
        raise Exception()

    def dump(self, i=0):
        """Print out a graphical representation of the tree."""
        print (" "*i)+str(self.__class__.__name__), self.weights
        for i1, i2, child in self.iter_childs():
            child.dump(i+2)

    @staticmethod
    def create_group(l):
        """Creates a group with childs $l$.

        pre:
            is_elementlist(l)
        """
        # should be overriden by derived classes
        return Group(l)



class GroupBase(Element):
    """Groups are optional elements which can be added to make the tree
    more efficient. They have a similar role as brackets in
    mathematics. Replacing a tree by its childs will not change any
    meaning.
    """
    is_group = True
    has_childs = True

    def compute_weights(self):
        if len(self.childs): # for empty groups, we use the default weights
            w_list = zip(*[child.weights for child in self.childs])
            self.weights = [f(l) for (l, f) in zip(w_list, self.functions)]

    def iter_childs(self):
        i1 = 0
        for child in self.childs:
            i2 = i1+len(child)
            yield i1, i2, child
            i1 = i2

    def replace_child(self, i1, i2, stuff):
        assert 0 <= i1 <= i2 <= len(self)
        stuff = filter(len, stuff) # XXX is necessary. But why do we
                                   # get empty groups?
        r1 = []
        r2 = []
        j1 = 0
        for item in self.childs:
            j2 = j1+len(item)
            if j2 <= i1:
                r1.append(item)
            elif j1 >= i2:
                r2.append(item)
            elif i1 <= j1 <= j2 <= i2:
                # Item is beeing replaced
                pass
            else:
                # Item overlaps i1, i2. This is not allowed!
                raise IndexError((i1, i2))
            j1 = j2
        # XXX TODO: remove debug code
        tmp = groups(join(r1, stuff, r2))
        try:
            assert calc_length(tmp) == len(self)+calc_length(stuff)-(i2-i1)
        except:
            dump(self, "self")
            print "len(tmp):", len(tmp)
            print "len(self):", len(self)
            print "i1, i2:", i1, i2
            dump_list(stuff)
            dump_list(tmp)
            raise
        return tmp

    def takeout(self, i1, i2):
        # Preconditions don't seem to be evaluated. Therefore we use
        # an additional assertion here.
        if debug > 1: # XXX remove this
            if not is_root_efficient(self):
                dump(self)
                print "nmax=", nmax
            assert is_root_efficient(self)

        if not (0 <= i1 <= i2 <= len(self)):
            raise IndexError([i1, i2])
        r1 = []; r2 = []; r3 = []; r4 = []
        k1 = []; k2 = []; k3 = []
        for j1, j2, child in self.iter_childs():
            if j2 <= i1:
                r1.append(child)
            elif i1 <= j1 <= j2 <= i2:
                k2.append(child)
            elif j1 <= i1 <= j2:
                r, k = child.takeout(max(i1-j1, 0), min(i2-j1, len(child)))
                r2.extend(r)
                k1.extend(k)
            elif j1 <= i2 <= j2:
                r, k = child.takeout(max(i1-j1, 0), min(i2-j1, len(child)))
                r3.extend(r)
                k3.extend(k)
            elif i2 <= j1:
                r4.append(child)
        # Note that we are returning a list of elements which have
        # been in the content before. Therefore, we are decreasing the
        # depth by 1 (or more). This is the reason why the returned
        # lists are list_efficient. The correctness of the approach
        # can be seen from full induction.
        return join(r1, r2, r3, r4), join(k1, k2, k3)

    def __repr__(self):
        return 'G(%s)' % repr(list(self.childs))


class Group(GroupBase):
    def __init__(self, items):
        self.childs = filter(len, items)
        if len(self.childs):            
            self.compute_weights()
G = Group



def homogenize(stuff):
    """Transforms $stuff$ into a homogeneous list of elements.

       The depth of the returned elements equals to the maxmimum depth
       in $stuff$.

       pre:
           is_elementlist(stuff)

       post[]:
           maxdepth(__return__) == mindepth(__return__)
    """
    l = []
    ll = [l]
    stuff = list(stuff[:])
    d = None
    while len(stuff):
        element = stuff[0]
        delement = depth(element)
        if delement == d or not l:
            l.append(element)
        else:
            l = [element]
            ll.append(l)
        d = delement
        stuff = stuff[1:]
    return join(*ll)



def simple_insert(element, i, stuff):
    """Inserts $stuff$ in $element$ at position $i$.

       All elements in stuff must have zero depth. This restriction makes
       it simpler and faster compared to the general insert-function.

       Returns a list of texels.

       pre:
           is_elementlist(stuff)

       post[stuff, element]:
           calc_length(__return__) == calc_length(stuff)+len(element)
    """
    assert type(stuff) is list
    if maxdepth(stuff) > 0:
        raise Exception("maxdepth(stuff) must be 0 for simple_insert")

    for i1, i2, child in element.iter_childs():
        if i1 <= i <= i2:
            new = simple_insert(child, i-i1, stuff)
            return element.replace_child(i1, i2, new)
    return element.replace_child(i, i, stuff)


def _join(l1, l2):
    """
    Joins the lists of elements $l1$ and $l2$.

    Given that l1 and l2 are each homogeneous then also the returned
    list is homogeneous. No healing is performed.

    pre:
        is_elementlist(l1)
        is_elementlist(l2)
        is_homogeneous(l1)
        is_homogeneous(l2)

    post[l1, l2]:
        is_homogeneous(__return__)
        calc_length(l1)+calc_length(l2) == calc_length(__return__)
        maxdepth(__return__) == max(maxdepth(l1), maxdepth(l2))
    """
    l1 = filter(len, l1) # strip off empty elements
    l2 = filter(len, l2) #
    if not l1:
        return l2
    if not l2:
        return l1
    t1 = l1[-1]
    t2 = l2[0]
    d1 = depth(t1)
    d2 = depth(t2)
    if d1 == d2:
        return l1+l2
    elif d1 > d2:
        return l1[:-1]+groups(_join(t1.childs, l2))
    # d1 < d2
    return groups(_join(l1, t2.childs))+l2[1:]

def join(*args):
    """Join several homogeneous lists of elements.

       The returned list is homogeneous.

       pre:
           forall(args, is_elementlist)
           forall(args, is_homogeneous)

       post[args]:
           is_homogeneous(__return__)
           sum([calc_length(x) for x in args]) == calc_length(__return__)
           #out("join check: ok")
    """
    return reduce(_join, args)


def insert(element, i, stuff):
    """Inserts the list $stuff$ at position $i$.

       Returns a list of elements. Unlike simple_insert, stuff can
       contain elements with depth>0.  Note that insert can increase
       the elements depth. The returned list is always homogeneous.

       pre:
           isinstance(element, Element)
           is_elementlist(stuff)
           is_homogeneous(stuff)

       post[element, stuff]:
           calc_length(__return__) == len(element)+calc_length(stuff)
           is_homogeneous(__return__)

    """
    assert 0 <= i <= len(element)
    if element.has_childs:
        k = -1
        for i1, i2, child in element.iter_childs():
            k += 1
            if i1 <= i <= i2:
                new = insert(child, i-i1, stuff)
                return element.replace_child(i1, i2, new)
    return join(copy(element, 0, i), stuff, copy(element, i, len(element)))



def groups(l):
    """Transform the list of texels $l$ into a list of groups.

       If texels have depth d, groups will have depth d+1.

       pre:
           is_elementlist(l)
           is_homogeneous(l)

       post[l]:
           is_homogeneous(__return__)
           calc_length(l) == calc_length(__return__)
           #out("groups check: ok")
    """
    r = []
    N = len(l)
    if N == 0:
        return r
    Group = l[0].create_group

    n = N // nmax
    if n*nmax < N:
        n = n+1
    rest = n*nmax-N

    a = nmax-float(rest)/n
    b = a
    while l:
        i = int(b+1e-10)
        r.append(Group(l[:i]))
        l = l[i:]
        b = b-i+a
    return r


def remove(root, i1, i2):
    """Removes all content of $root$ between $i1$ and $i2$.

       pre:
           isinstance(root, Element)

       post[root, i1, i2]:
           calc_length(__return__) == len(root)-(i2-i1)
    """
    return root.takeout(i1, i2)[0]


def copy(root, i1, i2):
    """Copy all content of $root$ between $i1$ and $i2$.

       pre:
           isinstance(root, Element)

       post[i1, i2]:
           calc_length(__return__) == (i2-i1)
    """
    return root.takeout(i1, i2)[1]


def ungroup(element):
    """Recursively removes all groups. Returns a list of texels.

       pre:
           isinstance(element, Element)

       post[element]:
           calc_length(__return__) == len(element)
           maxdepth(__return__) == 0
    """
    if is_group(element):
        l = []
        for child in element.childs:
            l.extend(ungroup(child))
        return l
    return [element]


def grouped(stuff):
    """Creates a single group from the list of elements $stuff$.

       If the number of elements exceeds nmax, subgroups are formed.
       Therefore, the depth can increase. Note that stuff is not
       allowed to be empty.

       pre:
           is_elementlist(stuff)
           is_homogeneous(stuff)
           len(stuff) > 0

       post[stuff]:
           len(__return__) == calc_length(stuff)
    """
    while len(stuff) > nmax:
        stuff = groups(stuff)
    g = stuff[0].create_group(stuff)
    return strip(g)

def _grouped(stuff):
    # For testing only.
    if not stuff:
        return Group([])
    return grouped(stuff)


def get_rightmost(element):
    """Returns the rightmost subelement of $element$."""
    if is_group(element):
        return get_rightmost(element.childs[-1])
    return element


def get_leftmost(element):
    """Returns the leftmost subelement of $element$."""
    if is_group(element):
        return get_leftmost(element.childs[0])
    return element


def exchange_rightmost(element, new):
    """Replace the rightmost subelement of $element$ by $new$."""
    if is_group(element):
        l = exchange_rightmost(element.childs[-1], new)
        return element.create_group(element.childs[:-1]+[l])
    return new


def exchange_leftmost(element, new):
    """Replace the lefttmost subelement of $element$ by $new$."""
    if is_group(element):
        l = exchange_leftmost(element.childs[0], new)
        return element.create_group([l]+element.childs[1:])
    return new


def remove_leftmost(element):
    """Removes the leftmost subelement of $element$.

       Note that this function can return an empty group.
    """
    if is_group(element):
        l = remove_leftmost(element.childs[0])
        return element.create_group([l]+element.childs[1:])
    return element.create_group([])



def _heal(element, i):
    """Recursively heal $element$ at position $i$.

       Algorithm:
       . iterate until i is at the edge of an element
       . find the leftmost and rightmost elements
       . if both can be merged, replace the left one by the merged pair
         and replace remove the right one

       pre:
           isinstance(element, Element)

       post[element]:
           calc_length(__return__) == len(element)
           extract_text(__return__) == extract_text(element)
    """
    n = len(element)
    if not element.has_childs:
        return [element]
    last_child = None
    l1 = None # indices of last texel
    l2 = None # 
    for i1, i2, child in element.iter_childs():
        if i1 == i and last_child is not None and i1 == l2:
            left = get_rightmost(last_child)
            right = get_leftmost(child)
            if left.can_merge(right):
                new = left.merge(right)
                l = [exchange_rightmost(last_child, new),
                     remove_leftmost(child)]
                return element.replace_child(l1, i2, l)

        elif i1 < i < i2:
            return element.replace_child(i1, i2, _heal(child, i-i1))
        last_child = child
        l1 = i1
        l2 = i2
    return [element]


def heal(element, *indices):
    """Heal element at the positions $indices$.

       pre:
           isinstance(element, Element)

       post[element]:
           len(__return__) == len(element)
           extract_text(__return__) == extract_text(element)
           depth(__return__) <= depth(element)
    """
    if not len(element): # necessary because grouped() can't handle empty
                         # elements
        return element
    n = len(element)
    for i in indices:
        element = grouped(_heal(element, i))
    return element


class Text(Element):
    # An element which holds string data (for testing)
    def __init__(self, text):
        self.text = text
        self.weights = (0, len(text))

    def can_merge(self, other):
        return isinstance(other, Text)

    def merge(self, other):
        return Text(self.text+other.text)

    def replace_child(self, i1, i2, stuff):
        r1 = Text(self.text[:i1])
        r2 = Text(self.text[i2:])
        l = []
        if len(r1):
            l.append(r1)
        l = join(l, stuff)
        if len(r2):
            l = join(l, [r2])
        return l

    def takeout(self, i1, i2):
        i1 = max(0, i1)
        i2 = min(len(self), i2)
        r1 = self.text[:i1]
        r2 = self.text[i2:]
        r3 = self.text[i1:i2]
        return [Text(r1+r2)], [Text(r3)]

    def __repr__(self):
        return "T(%s)" % repr(self.text)

    def dump(self, n=0):
        print (" "*n)+str(self), self.weights
T = Text


class TextNoMerge(Text):
    # Another class for testing.
    def can_merge(self, other):
        return False


def extract_text(element):
    # for testing
    if type(element) in (list, tuple):
        return u''.join([extract_text(x) for x in element])
    elif isinstance(element, Text):
        return element.text
    else:
        return u''.join([extract_text(x) for i1, i2, x in \
                         element.iter_childs()])
    return u''


if debug:
     #enable contract checking
     import contract
     contract.checkmod(__name__)
     print "enabled contracts"

def test_00():
    "homogenize"
    global debug
    debug = True
    assert str(homogenize([G([T('X')]), T('0'), T('1')])) == \
        "[G([T('X'), T('0'), T('1')])]"


def test_01():
    "is_homogeneous"
    assert not is_homogeneous([G([T('X')]), T('0'), T('1')])
    for x in G([T('X')]), G([T('0'), T('1')]):
        print depth(x), x
    assert is_homogeneous([G([T('X')]), G([T('0'), T('1')])])


def test_02():
    "strip"
    element = G([G([G([T('1'), T('2')]), G([T('3')])])])
    element.dump()
    print depth(element)
    assert depth(element) == 3
    assert depth(strip(element)) == 2
    #strip(element).dump()


def test_03():
    "is_root_efficient"
    assert is_root_efficient(G([T('1')]))
    assert not is_root_efficient(G([G([T('1')])]))
    set_nmax(3)
    assert is_root_efficient(G([T('1'), T('2'), T('3')]))
    set_nmax(2)
    #G([T('1'), T('2'), T('3')]).dump()
    assert not is_root_efficient(G([T('1'), T('2'), T('3')]))

def test_04():
    "insert"
    set_nmax(4)
    s = G([T('1'), T('2'), T('3')])
    element = G([s, s, s])

    tmp = insert(element, 3, [T('X'), T('Y')])
    assert depth(grouped(tmp)) == depth(element)
    assert extract_text(tmp) == '123XY123123'

    tmp = insert(element, 3, [G([T('X'), T('Y')])])
    assert depth(grouped(tmp)) == depth(element)
    assert extract_text(tmp) == '123XY123123'

    tmp = insert(grouped(tmp), 3, [G([T('Z'), T('#')])])
    assert extract_text(tmp) == '123Z#XY123123'


def test_05():
    "growth in insert"
    set_nmax(3)
    g = Group([Text("a"), Text("b"), Text("c")])
    assert depth(g) == 1
    assert repr(insert(g, 0, [Text("d")])) == \
        "[G([T('d'), T('a')]), G([T('b'), T('c')])]"


def test_06():
    "efficient insert / takeout"
    set_nmax(3)
    texel = Group([])
    import random
    for i in range(100):
        x = random.choice("abcdefghijklmnopqrstuvwxyz")
        tmp = insert(texel, i, [TextNoMerge(x)])
        assert is_list_efficient(tmp)
        texel = _grouped(tmp)
    assert is_root_efficient(texel)

    while len(texel):
        i1 = random.randrange(len(texel)+1)
        i2 = random.randrange(len(texel)+1)
        i1, i2 = sorted([i1, i2])
        r, k = texel.takeout(i1, i2)
        assert is_list_efficient(r)
        texel = _grouped(r)
    assert is_root_efficient(texel)


def test_07():
    "depth"
    assert depth(Text('abc')) == 0
    assert depth(G([Text('abc')])) == 1
    assert depth(G([])) == 0
    element = G([G([G([T('1'), T('2')]), G([T('3')])])])
    assert depth(element) == 3


# XXX TODO: add more tests




# -*- coding: latin-1 -*-




class Aborted(Exception):
    pass


class Client:
    name = 'unnamed' # each client should have a unique name 
    counter = 0
    can_abort = False # not all clients can abort

    def abort(self):
        pass

    def execute(self, texel):
        """Returns a tuple of completions. """
        pass

    def complete(self, word, nmax=None):
        """Returns a tuple of completions. """
        pass

    def colorize(self, texel):
        """Colorizes $texel$ to make it more readable. """
        return texel



class ClientPool:
    def __init__(self):
        self._clients = {}

    def register(self, client):
        self._clients[client.name] = client

    def get_matching(self, texel):
        return self._clients[texel.client_name]



# -*- coding: latin-1 -*-


class Properties(object):
    """
    This class implements the standard interface for initialising
    properties using keyword arguments.
    """

    def __init__(self, **kw):
        "Properties(name=value, ...) passes the given arguments to the set() method."
        self.set(**kw)

    def set(self, **kw):
        """set(name=value, ...) sets property values according to the given
        keyword arguments."""
        for name, value in kw.iteritems():
            try:
                getattr(self, 'set_' + name)(value)
            except AttributeError:
                # Der Setter existiert nicht. Gibt es vielleicht ein
                # passendes Propertyobjekt?
                try:
                    obj = getattr(self.__class__, name)
                except :
                    raise AttributeError, name
                obj.fset(self, value)




def overridable_property(name, doc = None):
    """Creates a property which calls methods get_xxx and set_xxx of
    the underlying object to get and set the property value, so that
    the property's behaviour may be easily overridden by subclasses."""

    getter_name = intern('get_' + name)
    setter_name = intern('set_' + name)
    return property(
        lambda self: getattr(self, getter_name)(),
        lambda self, value: getattr(self, setter_name)(value),
        None,
        doc)





# -*- coding: latin-1 -*-


import sys
sys.path.insert(0, '..')

from pynotebook.nbview import TextModel, WXTextView, Cell
from pynotebook.textformat import fromtext
import wx




examples = r"""[In 0]:
from pynotebook.nbtexels import BitmapRGB, BitmapRGBA

[In 1]:
def output(obj, iserr=False):
    if isinstance(obj, wx.Bitmap):
        im = obj.ConvertToImage()
        return __output__(
            BitmapRGBA(im.GetData(), im.GetAlphaData(), 
            im.GetSize()), iserr)
        
    return __output__(obj, iserr)
[In 2]:
import wx
bmp = wx.ArtProvider.GetBitmap(wx.ART_WARNING, size=(128, 128))

[In 3]:
bmp
"""

def demo_00():
    app = wx.App(redirect=False)
    model = fromtext(examples, Cell=Cell)

    frame = wx.Frame(None)
    win = wx.Panel(frame, -1)
    view = WXTextView(win, -1, style=wx.SUNKEN_BORDER)
    view.model = model
    box = wx.BoxSizer(wx.VERTICAL)
    box.Add(view, 1, wx.ALL|wx.GROW, 1)
    win.SetSizer(box)
    win.SetAutoLayout(True)

    frame.Show()

    if 0:
        from pynotebook.wxtextview import testing
        testing.pyshell(locals())
    app.MainLoop()


if __name__ == '__main__':
    demo_00()



#
#		Python GUI - View Base - Generic
#

from .properties import overridable_property

class ViewBase(object):
    """ViewBase is an abstract base class for user-defined views.
    It provides facilities for handling mouse and keyboard events
    and associating the view with one or more models, and default
    behaviour for responding to changes in the models."""

    models = overridable_property('models',
        "List of Models being observed. Do not modify directly.")

    model = overridable_property('model',
        "Convenience property for views which observe only one Model.")


    def __init__(self):
        self._models = []

    def destroy(self):
        #print "ViewBase.destroy:", self ###
        for m in self._models[:]:
            #print "ViewBase.destroy: removing model", m ###
            self.remove_model(m)

    #
    #		Getting properties
    #

    def get_model(self):
        models = self._models
        if models:
            return self._models[0]
        else:
            return None

    def get_models(self):
        return self._models

    #
    #		Setting properties
    #

    def set_model(self, new_model):
        models = self._models
        if not (len(models) == 1 and models[0] == new_model):
            for old_model in models[:]:
                self.remove_model(old_model)
            if new_model is not None: # XXX CHRIS. Vorher gabs Probleme mit __len__
                self.add_model(new_model)

    #
    #   Model association
    #

    def add_model(self, model):
        """Add the given Model to the set of models being observed."""
        if model not in self._models:
            self._models.append(model)
            model.add_view(self)

    def remove_model(self, model):
        """Remove the given Model from the set of models being observed."""
        if model in self._models:
            self._models.remove(model)
            model.remove_view(self)

    #
    #		Callbacks
    #

    def model_changed(self, model, *args, **kwds):
        """Default method called by the attached Model's notify_views
        method. Default is to date the whole view."""
        pass #self.invalidate()

    def model_destroyed(self, model):
        """Called when an attached model is destroyed."""
        pass




# -*- coding: latin-1 -*-


from .textmodel.container import Container
from .textmodel.textmodel import TextModel, dump_range
from .textmodel.texeltree import Texel, Glyph, NL

import wx



def mk_textmodel(texel):
    model = TextModel()
    model.texel = texel
    return model



class Cell(Container):
    # XXX currently there is only one cell type. In future we should
    # also have text cells or cells for different scripting
    # languages. We should have a generla ScriptCell.

    client_name = 'direct python'

    def __init__(self, input, output, number = 0, **kwargs):
        assert isinstance(input, Texel)
        assert isinstance(output, Texel)
        self.input = input
        self.output = output
        self.number = number
        Container.__init__(self, **kwargs)

    def get_empties(self):
        return NL, NL, NL

    def get_emptychars(self):
        return '\n\n\n'

    def get_content(self):
        return self.input, self.output

    def get_kwds(self):
        kwds = Container.get_kwds(self)
        kwds['number'] = self.number
        return kwds



class NotFound(Exception): pass

def find_cell(texel, i, i0=0):
    if isinstance(texel, Cell):
        return i0, texel
    elif texel.is_group:
        for j1, j2, child in texel.iter_childs():
            if j1<=i<j2:
                return find_cell(child, i-j1, i0+j1)
    raise NotFound()



class BitmapRGB(Glyph):
    def __init__(self, data, size):
        self.data = data
        self.size = size

    def __repr__(self):
        return 'BitmapRGB(...)'



class BitmapRGBA(Glyph):
    def __init__(self, data, alpha, size):
        self.data = data
        self.alpha = alpha
        self.size = size

    def __repr__(self):
        return 'BitmapRGB(...)'






# -*- coding: latin-1 -*-

from .testdevice import TESTDEVICE
from .boxes import TabulatorBox, TextBox, EmptyTextBox, NewlineBox, Row


def find_bestbreak(box, w):
    if not isinstance(box, TextBox):
        return 0
    text = box.text
    parts = box.measure_parts(text)

    # 1. try to break after a space
    for i, part in reversed(tuple(enumerate(parts))):
        if i>0 and part <= w and text[i] == ' ':
            return i+1

    # 2. brute force break
    for i, part in enumerate(parts):
        if part > w:
            return i
    return 0


def split_box(box, i):
    if not isinstance(box, TextBox):
        assert i == 0
        return EmptyTextBox(), box
    text = box.text
    style = box.style
    device = box.device
    b1 = box.__class__(text[:i], style, device)
    b2 = box.__class__(text[i:], style, device)
    return b1, b2


def simple_linewrap(boxes, maxw, tabstops=(), device=TESTDEVICE):
    assert isinstance(maxw, int)
    l = []
    rows = [l]
    w = 0
        
    boxes = list(boxes[:])
    last = None
    while boxes:
        box = boxes[0]
        boxes = boxes[1:]
        if not len(box):
            continue
        if w+box.width <= maxw:
            l.append(box)
            w += box.width
            if isinstance(box, TextBox) and ' ' in box.text:
                i = box.text.rindex(' ')+1
                last = len(l)-1, i
            continue
        # too wide -> break somewhere and start a new line
        if last:
            k, i = last
            lastbox = l[k]
            a, b = split_box(lastbox, i)
            assert len(a)+len(b) == len(lastbox)
            boxes = [b]+l[k+1:]+[box]+boxes
            del l[k:]            
            l.append(a)
        else:
            i = find_bestbreak(box, maxw-w)
            if i == 0:
                if l: # create new line
                    boxes = [box]+boxes
                else:
                    l.append(box)
            else:
                a, b = split_box(box, i)
                l.append(a)
                boxes = [b]+boxes
        w = 0
        l = []
        rows.append(l)
        last = None
    if not rows[-1]:
        rows = rows[:-1]
    return [Row(l, device) for l in rows]


def test_00():
    "find_bestbreak"
    box = TextBox("123 567 90")
    assert find_bestbreak(box, 3) == 3
    assert find_bestbreak(box, 4) == 4
    assert find_bestbreak(box, 5) == 4
    assert find_bestbreak(box, 6) == 4
    assert find_bestbreak(box, 7) == 4
    assert find_bestbreak(box, 8) == 8
    assert find_bestbreak(box, 9) == 8
    assert find_bestbreak(box, 10) == 8
    assert find_bestbreak(box, 11) == 8 # XXX Hmm?


def test_01():
    boxes = []
    for text in "aa bb cc dd ee".split():
        boxes.append(TextBox(text))
        if text == 'dd':
            boxes.append(NewlineBox())
    
    assert str(simple_linewrap(boxes, 5)) == \
        "[Row[TB('aa'), TB('bb'), TB('c')], Row[TB('c'), TB('dd'), NL, "\
        "TB('ee')]]"



# -*- coding: latin-1 -*-

# Some usefull functions to work with lists of objects
#
#
#
# Background: a little math of intervals
#
#   interval i: [i1, i2]
#   interval j: [j1, j2]
#
# No overlap:
#   i2 <= j1 or j2 <= i1
#
# i lies in j:
#   j1 <= i1 and j2 >= i2
#
# j lies in i:
#   i1 <= j1 and i2 >= j2
#
# i and j have a nonempty intersection
#   i1 < j2 and j1 < i2
#
#
# Overlap of i and j:
#    [max(i1, j1), min(i2, j2)]
#
# Size of overlap:
#    min(i2, j2) - max(i1, j1)


def calc_length(l):
    r = 0
    for item in l:
        r += len(item)
    return r


def iter_items(l):
    i1 = 0
    for item in l:
        i2 = i1+len(item)
        yield i1, i2, item
        i1 = i2


def intersecting_items(l, i1, i2):
    if not i2>i1:
        raise ValueError, "i2 must be larger i1"
    r = []
    j1 = 0
    for item in l:
        j2 = j1+len(item)
        if i1 < j2 and j1 < i2:
            r.append((j1, j2, item))
        j1 = j2
    return tuple(r)


def get_envelope(l, i1, i2):
    if len(l) == 0:
        return i1, i2 # XXX should i1, i2 be reduced to the existing
                      # interval?
    if i1 == i2:
        return get_interval(l, i1)
    if not i2 >= i1:
        raise ValueError("i2 must be >= i1")
    j1 = 0
    k1 = k2 = None
    for item in l:
        j2 = j1+len(item)
        if i1 < j2 and j1 < i2:
            if k1 is None:
                k1 = j1
                k2 = j2
            else:
                k2 = j2
        j1 = j2
    return k1, k2


def get_item(l, i):
    # Return element containing position i. If i is at a border
    # between to elements, return the element on the right.
    j1 = 0
    k1 = k2 = None
    items = []
    for item in l:
        j2 = j1+len(item)
        if i < j2 and j1 <= i: # nonempty intersection
            return j1, j2, item
        j1 = j2


def get_interval(l, i):
    # Returns interval of the elemnt holding position i. If i is at
    # the border between two elements, the right one os returner.

    j1 = 0
    k1 = k2 = None
    items = []
    for item in l:
        j2 = j1+len(item)
        if i < j2 and j1 <= i:
            return j1, j2
        j1 = j2
    return j1, j1


def get_items(l, i1, i2):
    if not i2>i1:
        raise ValueError("i2 must be larger i1")
    j1 = 0
    k1 = k2 = None
    items = []
    for item in l:
        j2 = j1+len(item)
        if i1 < j2 and j1 < i2:
            items.append(item)
            if k1 is None:
                k1 = j1
                k2 = j2
            else:
                k2 = j2
        j1 = j2
    return k1, k2, items


def replace(l, i1, i2, new):
    # Replace all elements between i1 and i2 with the elements in list
    # "new".
    #
    # -> replace(l, i1, i1, new) inserts at position i1
    # -> replace(l, length, length, new) appends
    # -> replace(l, i1, i2, []) removes from i1 to i2
    r1 = []
    r2 = []
    j1 = 0
    for item in l:
        j2 = j1+len(item)
        if j2 <= i1:
            r1.append(item)
        elif j1 >= i2:
            r2.append(item)
        j1 = j2
    return tuple(r1+list(new)+r2)


def listjoin(a, b, fun):
    if not len(a):
        return b
    if not len(b):
        return a
    return a[:-1]+fun(a[-1], b[0])+b[1:]




def test_00():
    assert get_envelope(['012345', '678', '9012345'], 6, 10) == (6, 16)
    assert get_envelope(['012345', '678', '9012345'], 6, 7) == (6, 9)


def test_01():
    assert replace(['012345', '678', '9012345'], 6, 9, []) == \
        ('012345', '9012345')

    assert replace(['012345', '678', '9012345'], 7, 7, []) == \
        ('012345', '9012345')


def test_02():
    assert intersecting_items(['012345', '678', '9012345'], 8, 9) == \
        ((6, 9, '678'),)
    assert intersecting_items(['012345', '678', '9012345'], 8, 10) == \
        ((6, 9, '678'), (9, 16, '9012345'),)



# -*- coding: latin-1 -*-

from .clients import Client, Aborted
from .nbstream import StreamRecorder

import sys
import traceback
import rlcompleter
import types



class FakeFile:
    encoding = 'UTF-8'
    def __init__(self, fun):
        self.fun = fun

    def write(self, s):
        self.fun(s)



class PythonClient(Client):
    name = 'direct python'
    can_abort = True
    aborted = False

    def __init__(self, namespace=None):
        if namespace is None:
            namespace = {}
        self.namespace = namespace
        self.init()

    def init(self):
        source = """
from pynotebook import nbtexels


def has_classname(obj, classname):
    "returns True if $obj$ is an instance of a class with name $classname$"
    s = "<class '%s'>" % classname
    try:
        return str(obj.__class__) == s
    except AttributeError:
        return False

def output(obj, iserr=False):
    __output__(__transform__(obj, iserr), iserr)

def __transform__(obj, iserr):
    if has_classname(obj, "matplotlib.figure.Figure"):
        obj.canvas.draw()
        data = obj.canvas.tostring_rgb()
        size = obj.canvas.get_width_height()
        return nbtexels.BitmapRGB(data, size)
        
    return obj
        """
        code = compile(source, "init", 'exec')
        ans = eval(code, self.namespace)
        self.namespace["ans"] = ans
        
    def abort(self):
        self.aborted = True

    def trace_fun(self, *args):
        if self.aborted:
            self.aborted = False
            raise Aborted()

    def execute(self, inputfield, output):
        source = inputfield.get_text()
        self.namespace['__output__'] = output
        self.counter += 1
        name = 'In[%s]' % self.counter
        bkstdout, bkstderr = sys.stdout, sys.stderr
        stdout = sys.stdout = FakeFile(lambda s:self.namespace['output'](s))
        stderr = sys.stderr = FakeFile(lambda s:self.namespace['output'](s, iserr=True))
        self.ok = False
        self.expression = False
        try:
            try:
                try:
                    code = compile(source, name, 'eval')
                    self.expression = True
                except SyntaxError:
                    sys.settrace(self.trace_fun)
                    code = compile(source, name, 'exec')
                ans = eval(code, self.namespace)
                self.namespace['ans'] = ans
                self.ok = True
            except Exception, e:
                self.show_traceback(name)
                self.namespace['ans'] = None
            if self.expression and self.ok:
                ans = self.namespace['ans']
                # Note that we do not output the repr() of ans but ans
                # itself. This allow us to do substitutions,
                # e.g. replace matplotlib figures by their graphical
                # representation.
                try:
                    self.namespace['output'](ans)
                except Exception, e:
                    self.show_traceback(name)
        finally:
            sys.stdout, sys.stderr = bkstdout, bkstderr
            sys.settrace(None)

    def show_syntaxerror(self, filename):
        # stolen from "idle" by  G. v. Rossum
        type, value, sys.last_traceback = sys.exc_info()
        sys.last_type = type
        sys.last_value = value
        if filename and type is SyntaxError:
            # Work hard to stuff the correct filename in the exception
            try:
                msg, (dummy_filename, lineno, offset, line) = value
            except:
                # Not the format we expect; leave it alone
                pass
            else:
                # Stuff in the right filename
                try:
                    # Assume SyntaxError is a class exception
                    value = SyntaxError(msg, (filename, lineno, offset, line))
                except:
                    # If that failed, assume SyntaxError is a string
                    value = msg, (filename, lineno, offset, line)

        info = traceback.format_exception_only(type, value)
        sys.stderr.write(''.join(info))

    def show_traceback(self, filename):
        if type(sys.exc_value) == types.InstanceType:
            args = sys.exc_value.args
        else:
            args = sys.exc_value

        traceback.print_tb(sys.exc_traceback.tb_next, None)
        self.show_syntaxerror(filename)  

    def complete(self, word, nmax=None):
        completer = rlcompleter.Completer(self.namespace)
        options = set()
        i = 0
        while True:
            option = completer.complete(word, i)
            i += 1
            if option is None or len(options) == nmax:
                break
            options.add(option)
        return options

    def colorize(self, inputtexel):
        text = inputtexel.get_text()

        # XXX The pycolorize function was only ment for benchmarking
        # the textmodel. Here, we should use an optimized variant
        # instead.
        from .textmodel.textmodel import pycolorize
        try:
            colorized = pycolorize(text).texel
        except Exception, e:
            return inputtexel
        assert len(colorized) == len(inputtexel)
        return colorized


def test_00():
    "execute"
    client = PythonClient()
    assert 'output' in client.namespace

    stream = StreamRecorder()
    client.execute("12+2", stream.output)
    assert client.namespace['ans'] == 14
    assert stream.messages == [(14, False)]

    stream = StreamRecorder()
    client.execute("12+(", stream.output)
    assert 'SyntaxError' in str(stream.messages)
    assert client.namespace['ans'] == None

    stream = StreamRecorder()
    client.execute("asdasds", stream.output)
    assert stream.messages == [
        ('  File "In[3]", line 1, in <module>\n', True), 
        ("NameError: name 'asdasds' is not defined\n", True)]

    stream = StreamRecorder()
    client.execute("a=1", stream.output)
    assert client.namespace['ans'] == None
    assert stream.messages == []

    stream = StreamRecorder()
    client.execute("a", stream.output)
    assert client.namespace['ans'] == 1
    assert stream.messages == [(1, False)]

    stream = StreamRecorder()
    client.execute("a+1", stream.output)
    assert client.namespace['ans'] == 2
    assert stream.messages == [(2, False)]

    stream = StreamRecorder()
    client.execute("print a", stream.output)
    assert stream.messages == [('1', False), ('\n', False)]
    
def test_01():
    "complete"
    client = PythonClient()
    assert client.complete('a') == ('abs(', 'all(', 'and', 'ans', 'any(', 
                                    'apply(', 'as', 'assert')
    assert client.complete('ba') == ('basestring(',)
    assert client.complete('cl') == ('class',)
    assert client.complete('class') == ('class', 'classmethod(')

def test_02():
    "abort"
    namespace = dict()
    client = PythonClient(namespace)
    stream = StreamRecorder()
    namespace['client'] = client
    client.execute("""
for i in range(10):
    print i
    if i>5:
        client.abort() # emulate a ctrl-c
    """, stream.output)
    assert 'Aborted' in str(stream.messages)
    
